@@ -3,10 +3,22 @@
 """Constants for PyKEEN."""
 
 from pathlib import Path
+from typing import Mapping
 
 import pystow
 import torch
 
+from .typing import (
+    COLUMN_HEAD,
+    COLUMN_RELATION,
+    COLUMN_TAIL,
+    LABEL_HEAD,
+    LABEL_RELATION,
+    LABEL_TAIL,
+    Target,
+    TargetColumn,
+)
+
 __all__ = [
     "PYKEEN_HOME",
     "PYKEEN_DATASETS",
@@ -44,3 +56,11 @@ DEFAULT_EMBEDDING_HPO_EMBEDDING_DIM_RANGE = dict(type=int, low=16, high=256, q=1
 USER_DEFINED_CODE = "<user defined>"
 
 AGGREGATIONS = {func.__name__: func for func in [torch.sum, torch.max, torch.mean, torch.logsumexp]}
+
+# TODO: extend to relation, cf. https://github.com/pykeen/pykeen/pull/728
+# SIDES: Tuple[Target, ...] = (LABEL_HEAD, LABEL_TAIL)
+TARGET_TO_INDEX: Mapping[Target, TargetColumn] = {
+    LABEL_HEAD: COLUMN_HEAD,
+    LABEL_RELATION: COLUMN_RELATION,
+    LABEL_TAIL: COLUMN_TAIL,
+}
