@@ -16,6 +16,8 @@ __all__ = [
 class NumericPathDataset(LazyDataset):
     """Contains a lazy reference to a training, testing, and validation dataset."""
 
+    triples_factory_cls = TriplesNumericLiteralsFactory
+
     def __init__(
         self,
         training_path: Union[str, pathlib.Path, TextIO],
@@ -46,12 +48,12 @@ class NumericPathDataset(LazyDataset):
             self._load_validation()
 
     def _load(self) -> None:
-        self._training = TriplesNumericLiteralsFactory(
+        self._training = self.triples_factory_cls.from_path(
             path=self.training_path,
             path_to_numeric_triples=self.literals_path,
             create_inverse_triples=self.create_inverse_triples,
         )
-        self._testing = TriplesNumericLiteralsFactory(
+        self._testing = self.triples_factory_cls.from_path(
             path=self.testing_path,
             path_to_numeric_triples=self.literals_path,
             entity_to_id=self._training.entity_to_id,  # share entity index with training
@@ -62,7 +64,7 @@ class NumericPathDataset(LazyDataset):
         # don't call this function by itself. assumes called through the `validation`
         # property and the _training factory has already been loaded
         assert self._training is not None
-        self._validation = TriplesNumericLiteralsFactory(
+        self._validation = self.triples_factory_cls.from_path(
             path=self.validation_path,
             path_to_numeric_triples=self.literals_path,
             entity_to_id=self._training.entity_to_id,  # share entity index with training
@@ -77,7 +79,9 @@ class NumericPathDataset(LazyDataset):
 
     def _summary_rows(self):
         rv = super()._summary_rows()
-        n_relations = len(self.training.literals_to_id)
-        n_triples = n_relations * self.training.num_entities
+        tf = self.training
+        assert isinstance(tf, TriplesNumericLiteralsFactory)
+        n_relations = len(tf.literals_to_id)
+        n_triples = n_relations * tf.num_entities
         rv.append(("Literals", "-", n_relations, n_triples))
         return rv
