@@ -7,20 +7,21 @@ import logging
 import timeit
 from abc import ABC, abstractmethod
 from contextlib import contextmanager
-from dataclasses import dataclass
 from math import ceil
 from textwrap import dedent
-from typing import Any, Collection, Iterable, List, Mapping, Optional, Tuple, Union, cast
+from typing import Any, ClassVar, Collection, Iterable, List, Mapping, Optional, Tuple, Type, Union, cast
 
 import numpy as np
+import pandas
 import torch
-from dataclasses_json import DataClassJsonMixin
 from tqdm.autonotebook import tqdm
 
+from ..constants import TARGET_TO_INDEX
+from ..metrics.utils import Metric
 from ..models import Model
 from ..triples.triples_factory import restrict_triples
 from ..triples.utils import get_entities, get_relations
-from ..typing import MappedTriples
+from ..typing import LABEL_HEAD, LABEL_RELATION, LABEL_TAIL, InductiveMode, MappedTriples, Target
 from ..utils import (
     format_relative_comparison,
     is_cuda_oom_error,
@@ -35,6 +36,7 @@ __all__ = [
     "MetricResults",
     "filter_scores_",
     "evaluate",
+    "prepare_filter_triples",
 ]
 
 logger = logging.getLogger(__name__)
@@ -42,6 +44,7 @@ logger = logging.getLogger(__name__)
 
 @contextmanager
 def optional_context_manager(condition, context_manager):
+    """Return an optional context manager based on the given condition."""
     if condition:
         with context_manager:
             yield context_manager
@@ -49,10 +52,22 @@ def optional_context_manager(condition, context_manager):
         yield
 
 
-@dataclass
-class MetricResults(DataClassJsonMixin):
+class MetricResults:
     """Results from computing metrics."""
 
+    metrics: ClassVar[Mapping[str, Type[Metric]]]
+
+    def __init__(self, data: Mapping):
+        """Initialize the result wrapper."""
+        self.data = data
+
+    def __getattr__(self, item):  # noqa:D105
+        # TODO remove this, it makes code much harder to reason about
+        if item not in self.data:
+            raise AttributeError
+        return self.data[item]
+
+    @abstractmethod
     def get_metric(self, name: str) -> float:
         """Get the given metric from the results.
 
@@ -61,6 +76,10 @@ class MetricResults(DataClassJsonMixin):
         """
         raise NotImplementedError
 
+    def to_dict(self):
+        """Get the results as a dictionary."""
+        return self.data
+
     def to_flat_dict(self) -> Mapping[str, Any]:
         """Get the results as a flattened dictionary."""
         return self.to_dict()
@@ -81,6 +100,7 @@ class Evaluator(ABC):
         batch_size: Optional[int] = None,
         slice_size: Optional[int] = None,
         automatic_memory_optimization: bool = True,
+        mode: Optional[InductiveMode] = None,
     ):
         """Initialize the evaluator.
 
@@ -90,12 +110,15 @@ class Evaluator(ABC):
         :param slice_size: >0. The divisor for the scoring function when using slicing
         :param automatic_memory_optimization: Whether to automatically optimize the sub-batch size during
             evaluation with regards to the hardware at hand.
+        :param mode:
+            the inductive mode, or None for transductive evaluation
         """
         self.filtered = filtered
         self.requires_positive_mask = requires_positive_mask
         self.batch_size = batch_size
         self.slice_size = slice_size
         self.automatic_memory_optimization = automatic_memory_optimization
+        self.mode = mode
 
     @classmethod
     def get_normalized_name(cls) -> str:
@@ -103,36 +126,21 @@ class Evaluator(ABC):
         return normalize_string(cls.__name__, suffix=Evaluator.__name__)
 
     @abstractmethod
-    def process_tail_scores_(
+    def process_scores_(
         self,
         hrt_batch: MappedTriples,
-        true_scores: torch.FloatTensor,
+        target: Target,
         scores: torch.FloatTensor,
+        true_scores: Optional[torch.FloatTensor] = None,
         dense_positive_mask: Optional[torch.FloatTensor] = None,
     ) -> None:
-        """Process a batch of triples with their computed tail scores for all entities.
+        """Process a batch of triples with their computed scores for all entities.
 
         :param hrt_batch: shape: (batch_size, 3)
-        :param true_scores: shape: (batch_size)
-        :param scores: shape: (batch_size, num_entities)
-        :param dense_positive_mask: shape: (batch_size, num_entities)
-            An optional binary (0/1) tensor indicating other true entities.
-        """
-        raise NotImplementedError
-
-    @abstractmethod
-    def process_head_scores_(
-        self,
-        hrt_batch: MappedTriples,
-        true_scores: torch.FloatTensor,
-        scores: torch.FloatTensor,
-        dense_positive_mask: Optional[torch.FloatTensor] = None,
-    ) -> None:
-        """Process a batch of triples with their computed head scores for all entities.
-
-        :param hrt_batch: shape: (batch_size, 3)
-        :param true_scores: shape: (batch_size)
+        :param target:
+            the prediction target
         :param scores: shape: (batch_size, num_entities)
+        :param true_scores: shape: (batch_size, 1)
         :param dense_positive_mask: shape: (batch_size, num_entities)
             An optional binary (0/1) tensor indicating other true entities.
         """
@@ -186,15 +194,15 @@ class Evaluator(ABC):
             model=model,
             additional_filter_triples=additional_filter_triples,
             mapped_triples=mapped_triples,
-            evaluators=self,
+            evaluator=self,
             batch_size=batch_size,
             slice_size=slice_size,
             device=device,
-            squeeze=True,
             use_tqdm=use_tqdm,
             tqdm_kwargs=tqdm_kwargs,
             restrict_entities_to=restrict_entities_to,
             do_time_consuming_checks=do_time_consuming_checks,
+            mode=self.mode,
         )
         # Since squeeze is true, we can expect that evaluate returns a MetricResult, but we need to tell MyPy that
         return cast(MetricResults, rv)
@@ -232,6 +240,8 @@ class Evaluator(ABC):
             Should a progress bar be displayed?
         :param restrict_entities_to:
             Whether to restrict the evaluation to certain entities of interest.
+        :param do_time_consuming_checks:
+            whether to perform time-consuming input validation for restricted evaluation
         :param additional_filter_triples:
             Additional true triples to filter out during filtered evaluation. Only needed if the evaluator is in
             filtered mode.
@@ -319,15 +329,15 @@ class Evaluator(ABC):
                     model=model,
                     additional_filter_triples=additional_filter_triples,
                     mapped_triples=mapped_triples,
-                    evaluators=self,
+                    evaluator=self,
                     only_size_probing=True,
                     device=device,
-                    squeeze=True,
                     use_tqdm=use_tqdm,
                     restrict_entities_to=restrict_entities_to,
                     do_time_consuming_checks=do_time_consuming_checks,
                     batch_size=values_dict.get("batch_size"),
                     slice_size=values_dict.get("slice_size"),
+                    mode=self.mode,
                 )
                 evaluated_once = True
             except RuntimeError as runtime_error:
@@ -416,6 +426,9 @@ def create_sparse_positive_filter_(
         - positives, shape: (2, m)
             The indices of positives in format [(batch_index, entity_id)].
         - the relation filter for re-usage.
+
+    :raises NotImplementedError:
+        if the `filter_col` is not in `{0, 2}`
     """
     if filter_col not in {0, 2}:
         raise NotImplementedError(
@@ -486,15 +499,46 @@ def filter_scores_(
     return scores
 
 
+def prepare_filter_triples(
+    mapped_triples: MappedTriples,
+    additional_filter_triples: Union[None, MappedTriples, List[MappedTriples]] = None,
+) -> MappedTriples:
+    """Prepare the filter triples from the evaluation triples, and additional filter triples."""
+    if additional_filter_triples is None:
+        logger.warning(
+            dedent(
+                """\
+            The filtered setting was enabled, but there were no `additional_filter_triples`
+            given. This means you probably forgot to pass (at least) the training triples. Try:
+
+                additional_filter_triples=[dataset.training.mapped_triples]
+
+            Or if you want to use the Bordes et al. (2013) approach to filtering, do:
+
+                additional_filter_triples=[
+                    dataset.training.mapped_triples,
+                    dataset.validation.mapped_triples,
+                ]
+        """
+            )
+        )
+        return mapped_triples
+
+    if torch.is_tensor(additional_filter_triples):
+        additional_filter_triples = [additional_filter_triples]
+
+    return torch.cat([*additional_filter_triples, mapped_triples], dim=0).unique(dim=0)
+
+
+# TODO: consider switching to torch.DataLoader where the preparation of masks/filter batches also takes place
 def evaluate(
     model: Model,
     mapped_triples: MappedTriples,
-    evaluators: Union[Evaluator, Collection[Evaluator]],
+    evaluator: Evaluator,
     only_size_probing: bool = False,
     batch_size: Optional[int] = None,
     slice_size: Optional[int] = None,
     device: Optional[torch.device] = None,
-    squeeze: bool = True,
     use_tqdm: bool = True,
     tqdm_kwargs: Optional[Mapping[str, str]] = None,
     restrict_entities_to: Optional[Collection[int]] = None,
@@ -502,7 +546,10 @@ def evaluate(
     do_time_consuming_checks: bool = True,
     additional_filter_triples: Union[None, MappedTriples, List[MappedTriples]] = None,
     pre_filtered_triples: bool = True,
-) -> Union[MetricResults, List[MetricResults]]:
+    targets: Collection[Target] = (LABEL_HEAD, LABEL_TAIL),
+    *,
+    mode: Optional[InductiveMode],
+) -> MetricResults:
     """Evaluate metrics for model on mapped triples.
 
     The model is used to predict scores for all tails and all heads for each triple. Subsequently, each abstract
@@ -516,8 +563,8 @@ def evaluate(
     :param mapped_triples:
         The triples on which to evaluate. The mapped triples should never contain inverse triples - these are created by
         the model class on the fly.
-    :param evaluators:
-        An evaluator or a list of evaluators working on batches of triples and corresponding scores.
+    :param evaluator:
+        The evaluator.
     :param only_size_probing:
         The evaluation is only performed for two batches to test the memory footprint, especially on GPUs.
     :param batch_size: >0
@@ -526,8 +573,6 @@ def evaluate(
         The divisor for the scoring function when using slicing.
     :param device:
         The device on which the evaluation shall be run. If None is given, use the model's device.
-    :param squeeze:
-        Return a single instance of :class:`MetricResults` if only one evaluator was given.
     :param use_tqdm:
         Should a progress bar be displayed?
     :param tqdm_kwargs:
@@ -552,12 +597,23 @@ def evaluate(
         Whether the triples have been pre-filtered to adhere to restrict_entities_to / restrict_relations_to. When set
         to True, and the triples have *not* been filtered, the results may be invalid. Pre-filtering the triples
         accelerates this method, and is recommended when evaluating multiple times on the same set of triples.
-    :param additional_filtered_triples:
-        Additional true triples to filter out during filtered evaluation.
-    """
-    if isinstance(evaluators, Evaluator):  # upgrade a single evaluator to a list
-        evaluators = [evaluators]
+    :param additional_filter_triples:
+        additional true triples to filter out during filtered evaluation.
+    :param targets:
+        the prediction targets
+    :param mode:
+        the inductive mode, or None for transductive evaluation
 
+    :raises NotImplementedError:
+        if relation prediction evaluation is requested
+    :raises ValueError:
+        if the pre_filtered_triples contain unwanted entities (can only be detected with the time-consuming checks).
+
+    :return:
+        the evaluation results
+    """
+    if LABEL_RELATION in targets:
+        raise NotImplementedError("cf. https://github.com/pykeen/pykeen/pull/728")
     start = timeit.default_timer()
 
     # verify that the triples have been filtered
@@ -597,43 +653,12 @@ def evaluate(
     # Ensure evaluation mode
     model.eval()
 
-    # Split evaluators into those which need unfiltered results, and those which require filtered ones
-    filtered_evaluators = list(filter(lambda e: e.filtered, evaluators))
-    unfiltered_evaluators = list(filter(lambda e: not e.filtered, evaluators))
-
-    # Check whether we need to be prepared for filtering
-    filtering_necessary = len(filtered_evaluators) > 0
-
-    # Check whether an evaluator needs access to the masks
-    # This can only be an unfiltered evaluator.
-    positive_masks_required = any(e.requires_positive_mask for e in unfiltered_evaluators)
-
     # Prepare for result filtering
-    if filtering_necessary or positive_masks_required:
-        if additional_filter_triples is None:
-            logger.warning(
-                dedent(
-                    """\
-                The filtered setting was enabled, but there were no `additional_filter_triples`
-                given. This means you probably forgot to pass (at least) the training triples. Try:
-
-                    additional_filter_triples=[dataset.training.mapped_triples]
-
-                Or if you want to use the Bordes et al. (2013) approach to filtering, do:
-
-                    additional_filter_triples=[
-                        dataset.training.mapped_triples,
-                        dataset.validation.mapped_triples,
-                    ]
-            """
-                )
-            )
-            all_pos_triples = mapped_triples
-        elif isinstance(additional_filter_triples, (list, tuple)):
-            all_pos_triples = torch.cat([*additional_filter_triples, mapped_triples], dim=0)
-        else:
-            all_pos_triples = torch.cat([additional_filter_triples, mapped_triples], dim=0)
-        all_pos_triples = all_pos_triples.to(device=device)
+    if evaluator.filtered or evaluator.requires_positive_mask:
+        all_pos_triples = prepare_filter_triples(
+            mapped_triples=mapped_triples,
+            additional_filter_triples=additional_filter_triples,
+        ).to(device=device)
     else:
         all_pos_triples = None
 
@@ -669,19 +694,17 @@ def evaluate(
         for batch in batches:
             batch_size = batch.shape[0]
             relation_filter = None
-            for column in (0, 2):
+            for target in targets:
                 relation_filter = _evaluate_batch(
-                    batch=batch,  # TODO fix typing
+                    batch=batch,
                     model=model,
-                    column=column,
-                    filtered_evaluators=filtered_evaluators,
-                    unfiltered_evaluators=unfiltered_evaluators,
+                    target=target,
+                    evaluator=evaluator,
                     slice_size=slice_size,
                     all_pos_triples=all_pos_triples,
                     relation_filter=relation_filter,
                     restrict_entities_to=restrict_entities_to,
-                    positive_masks_required=positive_masks_required,
-                    filtering_necessary=filtering_necessary,
+                    mode=mode,
                 )
 
             # If we only probe sizes we do not need more than one batch
@@ -694,7 +717,7 @@ def evaluate(
                 progress_bar.update(batch_size)
 
         # Finalize
-        results = [evaluator.finalize() for evaluator in evaluators]
+        result = evaluator.finalize()
 
     stop = timeit.default_timer()
     if only_size_probing:
@@ -702,38 +725,32 @@ def evaluate(
     else:
         logger.info("Evaluation took %.2fs seconds", stop - start)
 
-    if squeeze and len(results) == 1:
-        return results[0]
-
-    return results
+    return result
 
 
 def _evaluate_batch(
     batch: MappedTriples,
     model: Model,
-    column: int,
-    filtered_evaluators: Collection[Evaluator],
-    unfiltered_evaluators: Collection[Evaluator],
+    target: Target,
+    evaluator: Evaluator,
     slice_size: Optional[int],
     all_pos_triples: Optional[MappedTriples],
     relation_filter: Optional[torch.BoolTensor],
     restrict_entities_to: Optional[torch.LongTensor],
-    positive_masks_required: bool,
-    filtering_necessary: bool,
+    *,
+    mode: Optional[InductiveMode],
 ) -> torch.BoolTensor:
     """
-    Evaluate batch for all head predictions(column=0), or all tail predictions (column=2).
+    Evaluate ranking for batch.
 
     :param batch: shape: (batch_size, 3)
         The batch of currently evaluated triples.
     :param model:
         The model to evaluate.
-    :param column:
-        The column which to evaluate. Either 0 for head prediction, or 2 for tail prediction.
-    :param filtered_evaluators:
-        The evaluators which work on filtered scores.
-    :param unfiltered_evaluators:
-        The evaluators which work on unfiltered scores.
+    :param target:
+        The prediction target.
+    :param evaluator:
+        The evaluator
     :param slice_size:
         An optional slice size for computing the scores.
     :param all_pos_triples:
@@ -742,32 +759,19 @@ def _evaluate_batch(
         The relation filter. Can be re-used.
     :param restrict_entities_to:
         Restriction to evaluate only for these entities.
-    :param positive_masks_required:
-        Whether dense positive masks are required (by any unfiltered evaluator).
-    :param filtering_necessary:
-        Whether filtering is necessary.
+    :param mode:
+        the inductive mode, or None for transductive evaluation
+
+    :raises ValueError:
+        if all positive triples are required (either due to filtered evaluation, or requiring dense masks).
 
     :return:
         The relation filter, which can be re-used for the same batch.
     """
-    if column not in {0, 2}:
-        raise ValueError(f"column must be either 0 or 2, but is column={column}")
+    scores = model.predict(hrt_batch=batch, target=target, slice_size=slice_size, mode=mode)
 
-    # Predict scores once
-    if column == 2:  # tail scores
-        batch_scores_of_corrupted = model.predict_t(batch[:, 0:2], slice_size=slice_size)
-    else:
-        batch_scores_of_corrupted = model.predict_h(batch[:, 1:3], slice_size=slice_size)
-
-    # Select scores of true
-    batch_scores_of_true = batch_scores_of_corrupted[
-        torch.arange(0, batch.shape[0]),
-        batch[:, column],
-    ]
-
-    # Create positive filter for all corrupted
-    if filtering_necessary or positive_masks_required:
-        # Needs all positive triples
+    if evaluator.filtered or evaluator.requires_positive_mask:
+        column = TARGET_TO_INDEX[target]
         if all_pos_triples is None:
             raise ValueError(
                 "If filtering_necessary of positive_masks_required is True, all_pos_triples has to be "
@@ -781,63 +785,119 @@ def _evaluate_batch(
             relation_filter=relation_filter,
             filter_col=column,
         )
+    else:
+        positive_filter = relation_filter = None
+
+    if evaluator.filtered:
+        assert positive_filter is not None
+        # Select scores of true
+        true_scores = scores[torch.arange(0, batch.shape[0]), batch[:, column]]
+        # overwrite filtered scores
+        scores = filter_scores_(scores=scores, filter_batch=positive_filter)
+        # The scores for the true triples have to be rewritten to the scores tensor
+        scores[torch.arange(0, batch.shape[0]), batch[:, column]] = true_scores
+        # the rank-based evaluators needs the true scores with trailing 1-dim
+        true_scores = true_scores.unsqueeze(dim=-1)
+    else:
+        true_scores = None
 
     # Create a positive mask with the size of the scores from the positive filter
-    if positive_masks_required:
-        positive_mask = create_dense_positive_mask_(
-            zero_tensor=torch.zeros_like(batch_scores_of_corrupted),
-            filter_batch=positive_filter,
-        )
+    if evaluator.requires_positive_mask:
+        assert positive_filter is not None
+        positive_mask = create_dense_positive_mask_(zero_tensor=torch.zeros_like(scores), filter_batch=positive_filter)
     else:
         positive_mask = None
 
     # Restrict to entities of interest
     if restrict_entities_to is not None:
-        batch_scores_of_corrupted_ = batch_scores_of_corrupted[:, restrict_entities_to]
-        positive_mask = positive_mask[:, restrict_entities_to]
-    else:
-        batch_scores_of_corrupted_ = batch_scores_of_corrupted
+        scores = scores[:, restrict_entities_to]
+        if positive_mask is not None:
+            positive_mask = positive_mask[:, restrict_entities_to]
 
-    # Evaluate metrics on these *unfiltered* scores
-    for unfiltered_evaluator in unfiltered_evaluators:
-        if column == 2:  # tail scores
-            process = unfiltered_evaluator.process_tail_scores_
-        else:
-            process = unfiltered_evaluator.process_head_scores_
-        process(
-            hrt_batch=batch,
-            true_scores=batch_scores_of_true[:, None],
-            scores=batch_scores_of_corrupted_,
-            dense_positive_mask=positive_mask,
-        )
-
-    # Filter
-    if filtering_necessary:
-        batch_filtered_scores_of_corrupted = filter_scores_(
-            scores=batch_scores_of_corrupted,
-            filter_batch=positive_filter,
-        )
-
-        # The scores for the true triples have to be rewritten to the scores tensor
-        batch_filtered_scores_of_corrupted[
-            torch.arange(0, batch.shape[0]),
-            batch[:, column],
-        ] = batch_scores_of_true
-
-        # Restrict to entities of interest
-        if restrict_entities_to is not None:
-            batch_filtered_scores_of_corrupted = batch_filtered_scores_of_corrupted[:, restrict_entities_to]
-
-        # Evaluate metrics on these *filtered* scores
-        for filtered_evaluator in filtered_evaluators:
-            if column == 2:  # tail scores
-                process = filtered_evaluator.process_tail_scores_
-            else:
-                process = filtered_evaluator.process_head_scores_
-            process(
-                hrt_batch=batch,
-                true_scores=batch_scores_of_true[:, None],
-                scores=batch_filtered_scores_of_corrupted,
-            )
+    # process scores
+    evaluator.process_scores_(
+        hrt_batch=batch,
+        target=target,
+        true_scores=true_scores,
+        scores=scores,
+        dense_positive_mask=positive_mask,
+    )
 
     return relation_filter
+
+
+def get_candidate_set_size(
+    mapped_triples: MappedTriples,
+    restrict_entities_to: Optional[Collection[int]] = None,
+    restrict_relations_to: Optional[Collection[int]] = None,
+    additional_filter_triples: Union[None, MappedTriples, List[MappedTriples]] = None,
+    num_entities: Optional[int] = None,
+) -> pandas.DataFrame:
+    """
+    Calculate the candidate set sizes for head/tail prediction for the given triples.
+
+    :param mapped_triples: shape: (n, 3)
+        the evaluation triples
+    :param restrict_entities_to:
+        The entity IDs of interest. If None, defaults to all entities. cf. :func:`restrict_triples`.
+    :param restrict_relations_to:
+        The relations IDs of interest. If None, defaults to all relations. cf. :func:`restrict_triples`.
+    :param additional_filter_triples: shape: (n, 3)
+        additional filter triples besides the evaluation triples themselves. cf. `_prepare_filter_triples`.
+    :param num_entities:
+        the number of entities. If not given, this number is inferred from all triples
+
+    :return: columns: "index" | "head" | "relation" | "tail" | "head_candidates" | "tail_candidates"
+        a dataframe of all evaluation triples, with the number of head and tail candidates
+    """
+    # optionally restrict triples (nop if no restriction)
+    mapped_triples = restrict_triples(
+        mapped_triples=mapped_triples,
+        entities=restrict_entities_to,
+        relations=restrict_relations_to,
+    )
+
+    # evaluation triples as dataframe
+    columns = [LABEL_HEAD, LABEL_RELATION, LABEL_TAIL]
+    df_eval = pandas.DataFrame(
+        data=mapped_triples.numpy(),
+        columns=columns,
+    ).reset_index()
+
+    # determine filter triples
+    filter_triples = prepare_filter_triples(
+        mapped_triples=mapped_triples,
+        additional_filter_triples=additional_filter_triples,
+    )
+
+    # infer num_entities if not given
+    if restrict_entities_to:
+        num_entities = len(restrict_entities_to)
+    else:
+        # TODO: unique, or max ID + 1?
+        num_entities = num_entities or filter_triples[:, [0, 2]].view(-1).unique().numel()
+
+    # optionally restrict triples
+    filter_triples = restrict_triples(
+        mapped_triples=filter_triples,
+        entities=restrict_entities_to,
+        relations=restrict_relations_to,
+    )
+    df_filter = pandas.DataFrame(
+        data=filter_triples.numpy(),
+        columns=columns,
+    )
+
+    # compute candidate set sizes for different targets
+    # TODO: extend to relations?
+    for target in [LABEL_HEAD, LABEL_TAIL]:
+        total = num_entities
+        group_keys = [c for c in columns if c != target]
+        df_count = df_filter.groupby(by=group_keys).agg({target: "count"})
+        column = f"{target}_candidates"
+        df_count[column] = total - df_count[target]
+        df_count = df_count.drop(columns=target)
+        df_eval = pandas.merge(df_eval, df_count, on=group_keys, how="left")
+        df_eval[column] = df_eval[column].fillna(value=total)
+
+    return df_eval
