@@ -4,5 +4,5 @@
 
 from .cli import experiments
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     experiments()
