@@ -9,5 +9,5 @@ Why does this file exist, and why ``__main__``? For more info, read:
 
 from .cli import optimize
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     optimize()
