@@ -5,5 +5,5 @@
 from .builders import build_cli_from_cls
 
 __all__ = [
-    'build_cli_from_cls',
+    "build_cli_from_cls",
 ]
