@@ -4,22 +4,20 @@
 
 import logging
 from abc import ABC, abstractmethod
-from typing import Any, Mapping, Optional, Tuple, Union
+from typing import Any, Mapping, Optional, Sequence, Tuple, Union
 
 import torch
-from class_resolver import Resolver
-from class_resolver.api import Hint
+from class_resolver import ClassResolver, Hint, HintOrType, OptionalKwargs
+from class_resolver.contrib.torch import activation_resolver
 from torch import nn
 
-from .emb import EmbeddingSpecification, LowRankEmbeddingRepresentation, RepresentationModule
 from .init import uniform_norm_p1_
+from .representation import LowRankRepresentation, Representation
 from .weighting import EdgeWeighting, edge_weight_resolver
-from ..regularizers import Regularizer, regularizer_resolver
 from ..triples import CoreTriplesFactory
-from ..utils import activation_resolver
 
 __all__ = [
-    "RGCNRepresentations",
+    "RGCNRepresentation",
     "Decomposition",
     "BasesDecomposition",
     "BlockDecomposition",
@@ -177,6 +175,7 @@ class BasesDecomposition(Decomposition):
         :param memory_intense:
             Enable memory-intense forward pass which may be faster, in particular if the number of different relations
             is small.
+        :raises ValueError: If the ``num_bases`` is greater than ``num_relations``
         """
         super().__init__(
             input_dim=input_dim,
@@ -192,7 +191,7 @@ class BasesDecomposition(Decomposition):
         if num_bases > num_relations:
             raise ValueError("The number of bases should not exceed the number of relations.")
 
-        self.relation_representations = LowRankEmbeddingRepresentation(
+        self.relation_representations = LowRankRepresentation(
             max_id=num_relations,
             shape=(self.input_dim, self.output_dim),
             weight_initializer=uniform_norm_p1_,
@@ -212,7 +211,7 @@ class BasesDecomposition(Decomposition):
         :return:
             A 2-D matrix.
         """
-        return self.relation_representations(indices=relation_id).squeeze(dim=0)
+        return self.relation_representations(indices=torch.as_tensor(relation_id)).squeeze(dim=0)
 
     def _forward_memory_intense(
         self,
@@ -335,6 +334,7 @@ class BlockDecomposition(Decomposition):
             The number of blocks to use. Has to be a divisor of input_dim.
         :param output_dim: >0
             The output dimension. If None is given, defaults to input_dim.
+        :raises NotImplementedError: If ``input_dim`` is not divisible by ``num_blocks``
         """
         super().__init__(
             input_dim=input_dim,
@@ -557,10 +557,10 @@ class RGCNLayer(nn.Module):
         return y
 
 
-decomposition_resolver = Resolver.from_subclasses(base=Decomposition, default=BasesDecomposition)
+decomposition_resolver = ClassResolver.from_subclasses(base=Decomposition, default=BasesDecomposition)
 
 
-class RGCNRepresentations(RepresentationModule):
+class RGCNRepresentation(Representation):
     r"""Entity representations enriched by R-GCN.
 
     The GCN employed by the entity encoder is adapted to include typed edges.
@@ -589,7 +589,10 @@ class RGCNRepresentations(RepresentationModule):
     def __init__(
         self,
         triples_factory: CoreTriplesFactory,
-        embedding_specification: EmbeddingSpecification,
+        max_id: Optional[int] = None,
+        shape: Optional[Sequence[int]] = None,
+        entity_representations: HintOrType[Representation] = None,
+        entity_representations_kwargs: OptionalKwargs = None,
         num_layers: int = 2,
         use_bias: bool = True,
         activation: Hint[nn.Module] = None,
@@ -599,15 +602,21 @@ class RGCNRepresentations(RepresentationModule):
         edge_weighting: Hint[EdgeWeighting] = None,
         decomposition: Hint[Decomposition] = None,
         decomposition_kwargs: Optional[Mapping[str, Any]] = None,
-        regularizer: Hint[Regularizer] = None,
-        regularizer_kwargs: Optional[Mapping[str, Any]] = None,
+        **kwargs,
     ):
         """Instantiate the R-GCN encoder.
 
         :param triples_factory:
             The triples factory holding the training triples used for message passing.
-        :param embedding_specification:
-            The base embedding specification.
+        :param max_id:
+            The maximum number of IDs. could either be None (the default), or match the triples factory's number of
+            entities.
+        :param shape:
+            the shape information. If None, will propagate the shape information of the base entity representations.
+        :param entity_representations:
+            the base entity representations (or a hint for them)
+        :param entity_representations_kwargs:
+            additional keyword-based parameters for the base entity representations
         :param num_layers:
             The number of layers.
         :param use_bias:
@@ -626,13 +635,22 @@ class RGCNRepresentations(RepresentationModule):
             The decomposition, cf. :class:`pykeen.nn.message_passing.Decomposition`.
         :param decomposition_kwargs:
             Additional keyword based arguments passed to the decomposition upon instantiation.
-        :param regularizer:
-            A regularizer, which is applied to the selected embeddings in forward pass
-        :param regularizer_kwargs:
-            Additional keyword arguments passed to the regularizer
+        :param kwargs:
+            additional keyword-based parameters passed to :meth:`Representation.__init__`
+        :raises ValueError: If the triples factory creates inverse triples.
         """
-        base_embeddings = embedding_specification.make(num_embeddings=triples_factory.num_entities)
-        super().__init__(max_id=triples_factory.num_entities, shape=base_embeddings.shape)
+        if max_id:
+            assert max_id == triples_factory.num_entities
+
+        # has to be imported now to avoid cyclic imports
+        from . import representation_resolver
+
+        base_embeddings = representation_resolver.make(
+            entity_representations,
+            max_id=triples_factory.num_entities,
+            pos_kwargs=entity_representations_kwargs,
+        )
+        super().__init__(max_id=base_embeddings.max_id, shape=shape or base_embeddings.shape, **kwargs)
         self.entity_embeddings = base_embeddings
 
         if triples_factory.create_inverse_triples:
@@ -674,10 +692,6 @@ class RGCNRepresentations(RepresentationModule):
         # buffering of enriched representations
         self.enriched_embeddings = None
 
-        if regularizer is not None:
-            regularizer = regularizer_resolver.make(regularizer, pos_kwargs=regularizer_kwargs)
-        self.regularizer = regularizer
-
     def post_parameter_update(self) -> None:  # noqa: D102
         super().post_parameter_update()
 
@@ -693,7 +707,7 @@ class RGCNRepresentations(RepresentationModule):
             elif any(p.requires_grad for p in m.parameters()):
                 logger.warning("Layers %s has parameters, but no reset_parameters.", m)
 
-    def _real_forward(self) -> torch.FloatTensor:
+    def _real_forward_all(self) -> torch.FloatTensor:
         if self.enriched_embeddings is not None:
             return self.enriched_embeddings
 
@@ -738,14 +752,12 @@ class RGCNRepresentations(RepresentationModule):
 
         return x
 
-    def forward(
+    def _plain_forward(
         self,
         indices: Optional[torch.LongTensor] = None,
     ) -> torch.FloatTensor:
         """Enrich the entity embeddings of the decoder using R-GCN message propagation."""
-        x = self._real_forward()
+        x = self._real_forward_all()
         if indices is not None:
             x = x[indices]
-        if self.regularizer is not None:
-            self.regularizer.update(x)
         return x
