@@ -13,8 +13,10 @@ from operator import itemgetter
 from typing import (
     Any,
     Callable,
+    ClassVar,
     Generic,
     Iterable,
+    List,
     Mapping,
     MutableMapping,
     Optional,
@@ -26,23 +28,27 @@ from typing import (
 )
 
 import more_itertools
+import numpy
 import torch
-from class_resolver import Resolver
+from class_resolver import ClassResolver, Hint, OptionalKwargs
+from class_resolver.contrib.torch import activation_resolver
 from docdata import parse_docdata
 from torch import FloatTensor, nn
 from torch.nn.init import xavier_normal_
 
 from . import functional as pkf
 from .combinations import Combination
-from ..typing import HeadRepresentation, HintOrType, Initializer, RelationRepresentation, Sign, TailRepresentation
-from ..utils import (
-    CANONICAL_DIMENSIONS,
-    activation_resolver,
-    convert_to_canonical_shape,
-    ensure_tuple,
-    unpack_singletons,
-    upgrade_to_sequence,
+from .init import initializer_resolver
+from ..typing import (
+    HeadRepresentation,
+    HintOrType,
+    Initializer,
+    RelationRepresentation,
+    Representation,
+    Sign,
+    TailRepresentation,
 )
+from ..utils import ensure_tuple, unpack_singletons, upgrade_to_sequence
 
 __all__ = [
     "interaction_resolver",
@@ -66,6 +72,7 @@ __all__ = [
     "ERMLPEInteraction",
     "HolEInteraction",
     "KG2EInteraction",
+    "MultiLinearTuckerInteraction",
     "MuREInteraction",
     "NTNInteraction",
     "PairREInteraction",
@@ -90,25 +97,45 @@ logger = logging.getLogger(__name__)
 
 
 def parallel_slice_batches(
-    z: Union[FloatTensor, Tuple[FloatTensor, ...]],
-    slice_size: int,
+    *representations: Representation,
+    split_size: int,
     dim: int,
-) -> Iterable[Union[FloatTensor, Tuple[FloatTensor, ...]]]:
-    # input normalization -> Tuple[Tensor]
-    # shape: (num_representations,)
-    z_tup: Iterable[FloatTensor] = ensure_tuple(z)[0]
+) -> Iterable[Sequence[Representation]]:
+    """
+    Slice representations along the given dimension.
 
-    # split each representation along the given dimension
-    # shape: (num_representations,)
-    tuple_of_batches: Iterable[Sequence[FloatTensor]] = (zz.split(slice_size, dim=dim) for zz in z_tup)
+    :param representations:
+        the representations to slice
+    :param split_size:
+        the slice size
+    :param dim:
+        the dimension along which to slice
 
-    # create batches, shape: (num_slices,), each being a tuple of shape (num_representations,)
-    batch_of_tuples: Iterable[Tuple[FloatTensor]] = zip(*tuple_of_batches)
+    :yields: batches of sliced representations
+    """
+    # normalize input
+    rs: Sequence[Sequence[torch.FloatTensor]] = ensure_tuple(*representations)
+    # get number of head/relation/tail representations
+    length = list(map(len, rs))
+    splits = numpy.cumsum([0] + length)
+    # flatten list
+    rsl: Sequence[torch.FloatTensor] = sum(map(list, rs), [])
+    # split tensors
+    parts = [r.split(split_size, dim=dim) for r in rsl]
+    # broadcasting
+    n_parts = max(map(len, parts))
+    parts = [r_parts if len(r_parts) == n_parts else r_parts * n_parts for r_parts in parts]
+    # yield batches
+    for batch in zip(*parts):
+        # complex typing
+        yield unpack_singletons(*(batch[start:stop] for start, stop in zip(splits, splits[1:])))  # type: ignore
 
-    # unpack_singletons
-    batch_of_unpacked_tuples = unpack_singletons(*batch_of_tuples)
 
-    yield from batch_of_unpacked_tuples
+def parallel_unsqueeze(x: Representation, dim: int) -> Representation:
+    """Unsqueeze all representations along the given dimension."""
+    xs: Sequence[torch.FloatTensor] = upgrade_to_sequence(x)
+    xs = [xx.unsqueeze(dim=dim) for xx in xs]
+    return xs[0] if len(xs) == 1 else xs
 
 
 class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation, TailRepresentation], ABC):
@@ -117,12 +144,51 @@ class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation,
     #: The symbolic shapes for entity representations
     entity_shape: Sequence[str] = ("d",)
 
-    #: The symbolic shapes for entity representations for tail entities, if different. This is ony relevant for ConvE.
-    tail_entity_shape: Optional[Sequence[str]] = None
+    #: The symbolic shapes for entity representations for tail entities, if different.
+    #: Otherwise, the entity_shape is used for head & tail entities
+    _tail_entity_shape: Optional[Sequence[str]] = None
 
     #: The symbolic shapes for relation representations
     relation_shape: Sequence[str] = ("d",)
 
+    # if the interaction function's head parameter should only receive a subset of entity representations
+    _head_indices: Optional[Sequence[int]] = None
+
+    # if the interaction function's tail parameter should only receive a subset of entity representations
+    _tail_indices: Optional[Sequence[int]] = None
+
+    @property
+    def tail_entity_shape(self) -> Sequence[str]:
+        """Return the symbolic shape for tail entity representations."""
+        if self._tail_entity_shape is None:
+            return self.entity_shape
+        return self._tail_entity_shape
+
+    def head_indices(self) -> Sequence[int]:
+        """Return the entity representation indices used for the head representations."""
+        if self._head_indices is None:
+            return list(range(len(self.entity_shape)))
+        return self._head_indices
+
+    def tail_indices(self) -> Sequence[int]:
+        """Return the entity representation indices used for the tail representations."""
+        if self._tail_indices is None:
+            return list(range(len(self.tail_entity_shape)))
+        return self._tail_indices
+
+    def full_entity_shapes(self) -> Sequence[str]:
+        """Return all entity shapes (head & tail)."""
+        shapes: List[Optional[str]] = [None] * (max(itt.chain(self.head_indices(), self.tail_indices())) + 1)
+        for hi, hs in zip(self.head_indices(), self.entity_shape):
+            shapes[hi] = hs
+        for ti, ts in zip(self.tail_indices(), self.tail_entity_shape):
+            if shapes[ti] is not None and ts != shapes[ti]:
+                raise ValueError("Shape conflict.")
+            shapes[ti] = ts
+        if None in shapes:
+            raise AssertionError("Unused shape.")
+        return cast(List[str], shapes)
+
     @classmethod
     def get_dimensions(cls) -> Set[str]:
         """Get all of the relevant dimension keys.
@@ -132,7 +198,8 @@ class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation,
 
         :returns: a set of strings representting the dimension keys.
         """
-        return set(itt.chain(cls.entity_shape, cls.tail_entity_shape or set(), cls.relation_shape))
+        # TODO: cannot cover dynamic shapes, e.g., AutoSF
+        return set(itt.chain(cls.entity_shape, cls._tail_entity_shape or set(), cls.relation_shape))
 
     @abstractmethod
     def forward(
@@ -143,14 +210,14 @@ class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation,
     ) -> torch.FloatTensor:
         """Compute broadcasted triple scores given broadcasted representations for head, relation and tails.
 
-        :param h: shape: (batch_size, num_heads, 1, 1, ``*``)
+        :param h: shape: (`*batch_dims`, `*dims`)
             The head representations.
-        :param r: shape: (batch_size, 1, num_relations, 1, ``*``)
+        :param r: shape: (`*batch_dims`, `*dims`)
             The relation representations.
-        :param t: shape: (batch_size, 1, 1, num_tails, ``*``)
+        :param t: shape: (`*batch_dims`, `*dims`)
             The tail representations.
 
-        :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        :return: shape: batch_dims
             The scores.
         """
 
@@ -160,147 +227,39 @@ class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation,
         r: RelationRepresentation,
         t: TailRepresentation,
         slice_size: Optional[int] = None,
-        slice_dim: Optional[str] = None,
+        slice_dim: int = 1,
     ) -> torch.FloatTensor:
         """Compute broadcasted triple scores with optional slicing.
 
         .. note ::
             At most one of the slice sizes may be not None.
 
-        :param h: shape: (batch_size, num_heads, `1, 1, `*``)
+        # TODO: we could change that to slicing along multiple dimensions, if necessary
+
+        :param h: shape: (`*batch_dims`, `*dims`)
             The head representations.
-        :param r: shape: (batch_size, 1, num_relations, 1, ``*``)
+        :param r: shape: (`*batch_dims`, `*dims`)
             The relation representations.
-        :param t: shape: (batch_size, 1, 1, num_tails, ``*``)
+        :param t: shape: (`*batch_dims`, `*dims`)
             The tail representations.
         :param slice_size:
             The slice size.
         :param slice_dim:
-            The dimension along which to slice. From {"h", "r", "t"}
+            The dimension along which to slice. From {0, ..., len(batch_dims)}
 
-        :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        :return: shape: batch_dims
             The scores.
         """
-        return self._forward_slicing_wrapper(h=h, r=r, t=t, slice_size=slice_size, slice_dim=slice_dim)
-
-    def _score(
-        self,
-        h: HeadRepresentation,
-        r: RelationRepresentation,
-        t: TailRepresentation,
-        slice_size: Optional[int] = None,
-        slice_dim: str = None,
-    ) -> torch.FloatTensor:
-        """Compute scores for the score_* methods outside of models.
-
-        TODO: merge this with the Model utilities?
-
-        :param h: shape: (b, h, *)
-        :param r: shape: (b, r, *)
-        :param t: shape: (b, t, *)
-        :param slice_size:
-            The slice size.
-        :param slice_dim:
-            The dimension along which to slice. From {"h", "r", "t"}
-        :return: shape: (b, h, r, t)
-        """
-        args = []
-        for key, x in zip("hrt", (h, r, t)):
-            value = []
-            for xx in upgrade_to_sequence(x):  # type: torch.FloatTensor
-                # bring to (b, n, *)
-                xx = xx.unsqueeze(dim=1 if key != slice_dim else 0)
-                # bring to (b, h, r, t, *)
-                xx = convert_to_canonical_shape(
-                    x=xx,
-                    dim=key,
-                    num=xx.shape[1],
-                    batch_size=xx.shape[0],
-                    suffix_shape=xx.shape[2:],
-                )
-                value.append(xx)
-            # unpack singleton
-            if len(value) == 1:
-                value = value[0]
-            args.append(value)
-        h, r, t = cast(Tuple[HeadRepresentation, RelationRepresentation, TailRepresentation], args)
-        return self._forward_slicing_wrapper(h=h, r=r, t=t, slice_dim=slice_dim, slice_size=slice_size)
-
-    def _forward_slicing_wrapper(
-        self,
-        h: Union[torch.FloatTensor, Tuple[torch.FloatTensor, ...]],
-        r: Union[torch.FloatTensor, Tuple[torch.FloatTensor, ...]],
-        t: Union[torch.FloatTensor, Tuple[torch.FloatTensor, ...]],
-        slice_size: Optional[int],
-        slice_dim: Optional[str],
-    ) -> torch.FloatTensor:
-        """Compute broadcasted triple scores with optional slicing for representations in canonical shape.
-
-        .. note ::
-            Depending on the interaction function, there may be more than one representation for h/r/t. In that case,
-            a tuple of at least two tensors is passed.
-
-        :param h: shape: (batch_size, num_heads, 1, 1, ``*``)
-            The head representations.
-        :param r: shape: (batch_size, 1, num_relations, 1, ``*``)
-            The relation representations.
-        :param t: shape: (batch_size, 1, 1, num_tails, ``*``)
-            The tail representations.
-        :param slice_size:
-            The slice size.
-        :param slice_dim:
-            The dimension along which to slice. From {"h", "r", "t"}
-
-        :return: shape: (batch_size, num_heads, num_relations, num_tails)
-            The scores.
-
-        :raises ValueError:
-            If slice_dim is invalid.
-        """
         if slice_size is None:
-            scores = self(h=h, r=r, t=t)
-        elif slice_dim == "h":
-            dim = CANONICAL_DIMENSIONS[slice_dim]
-            scores = torch.cat(
-                [
-                    self(h=h_batch, r=r, t=t)
-                    for h_batch in parallel_slice_batches(
-                        z=h,
-                        slice_size=slice_size,
-                        dim=dim,
-                    )
-                ],
-                dim=dim,
-            )
-        elif slice_dim == "r":
-            dim = CANONICAL_DIMENSIONS[slice_dim]
-            scores = torch.cat(
-                [
-                    self(h=h, r=r_batch, t=t)
-                    for r_batch in parallel_slice_batches(
-                        z=r,
-                        slice_size=slice_size,
-                        dim=dim,
-                    )
-                ],
-                dim=dim,
-            )
-        elif slice_dim == "t":
-            dim = CANONICAL_DIMENSIONS[slice_dim]
-            scores = torch.cat(
-                [
-                    self(h=h, r=r, t=t_batch)
-                    for t_batch in parallel_slice_batches(
-                        z=t,
-                        slice_size=slice_size,
-                        dim=dim,
-                    )
-                ],
-                dim=dim,
-            )
-        else:
-            raise ValueError(f"Invalid slice_dim: {slice_dim}")
-        return scores
+            return self(h=h, r=r, t=t)
+
+        return torch.cat(
+            [
+                self(h=h_batch, r=r_batch, t=t_batch)
+                for h_batch, r_batch, t_batch in parallel_slice_batches(h, r, t, split_size=slice_size, dim=slice_dim)
+            ],
+            dim=slice_dim,
+        )
 
     def score_hrt(
         self,
@@ -320,7 +279,7 @@ class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation,
         :return: shape: (batch_size, 1)
             The scores.
         """
-        return self._score(h=h, r=r, t=t)[:, 0, 0, 0, None]
+        return self.score(h=h, r=r, t=t).unsqueeze(dim=-1)
 
     def score_h(
         self,
@@ -343,7 +302,12 @@ class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation,
         :return: shape: (batch_size, num_entities)
             The scores.
         """
-        return self._score(h=all_entities, r=r, t=t, slice_dim="h", slice_size=slice_size)[:, :, 0, 0]
+        return self.score(
+            h=parallel_unsqueeze(all_entities, dim=0),
+            r=parallel_unsqueeze(r, dim=1),
+            t=parallel_unsqueeze(t, dim=1),
+            slice_size=slice_size,
+        )
 
     def score_r(
         self,
@@ -366,7 +330,12 @@ class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation,
         :return: shape: (batch_size, num_entities)
             The scores.
         """
-        return self._score(h=h, r=all_relations, t=t, slice_dim="r", slice_size=slice_size)[:, 0, :, 0]
+        return self.score(
+            h=parallel_unsqueeze(h, dim=1),
+            r=parallel_unsqueeze(all_relations, dim=0),
+            t=parallel_unsqueeze(t, dim=1),
+            slice_size=slice_size,
+        )
 
     def score_t(
         self,
@@ -389,7 +358,12 @@ class Interaction(nn.Module, Generic[HeadRepresentation, RelationRepresentation,
         :return: shape: (batch_size, num_entities)
             The scores.
         """
-        return self._score(h=h, r=r, t=all_entities, slice_dim="t", slice_size=slice_size)[:, 0, 0, :]
+        return self.score(
+            h=parallel_unsqueeze(h, dim=1),
+            r=parallel_unsqueeze(r, dim=1),
+            t=parallel_unsqueeze(all_entities, dim=0),
+            slice_size=slice_size,
+        )
 
     def reset_parameters(self):
         """Reset parameters the interaction function may have."""
@@ -443,14 +417,14 @@ class LiteralInteraction(
     ) -> torch.FloatTensor:
         """Compute broadcasted triple scores given broadcasted representations for head, relation and tails.
 
-        :param h: shape: (batch_size, num_heads, 1, 1, ``*``)
+        :param h: shape: (`*batch_dims`, `*dims`)
             The head representations.
-        :param r: shape: (batch_size, 1, num_relations, 1, ``*``)
+        :param r: shape: (`*batch_dims`, `*dims`)
             The relation representations.
-        :param t: shape: (batch_size, 1, 1, num_tails, ``*``)
+        :param t: shape: (`*batch_dims`, `*dims`)
             The tail representations.
 
-        :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        :return: shape: batch_dims
             The scores.
         """
         # alternate way of combining entity embeddings + literals
@@ -477,14 +451,14 @@ class FunctionalInteraction(Interaction, Generic[HeadRepresentation, RelationRep
     ) -> torch.FloatTensor:
         """Compute broadcasted triple scores given broadcasted representations for head, relation and tails.
 
-        :param h: shape: (batch_size, num_heads, 1, 1, ``*``)
+        :param h: shape: (`*batch_dims`, `*dims`)
             The head representations.
-        :param r: shape: (batch_size, 1, num_relations, 1, ``*``)
+        :param r: shape: (`*batch_dims`, `*dims`)
             The relation representations.
-        :param t: shape: (batch_size, 1, 1, num_tails, ``*``)
+        :param t: shape: (`*batch_dims`, `*dims`)
             The tail representations.
 
-        :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        :return: shape: batch_dims
             The scores.
         """
         return self.__class__.func(**self._prepare_for_functional(h=h, r=r, t=t))
@@ -630,7 +604,8 @@ class ConvEInteraction(
     .. seealso:: :func:`pykeen.nn.functional.conve_interaction`
     """
 
-    tail_entity_shape = ("d", "k")  # with k=1
+    # vector & scalar offset
+    tail_entity_shape = ("d", "")
 
     #: The head-relation encoder operating on 2D "images"
     hr2d: nn.Module
@@ -947,7 +922,7 @@ class ProjEInteraction(FunctionalInteraction[FloatTensor, FloatTensor, FloatTens
         self.b_c = nn.Parameter(torch.empty(embedding_dim), requires_grad=True)
 
         # Global combination bias
-        self.b_p = nn.Parameter(torch.empty(1), requires_grad=True)
+        self.b_p = nn.Parameter(torch.empty(tuple()), requires_grad=True)
 
         if inner_non_linearity is None:
             inner_non_linearity = nn.Tanh()
@@ -1005,6 +980,11 @@ class TuckerInteraction(FunctionalInteraction[FloatTensor, FloatTensor, FloatTen
 
     func = pkf.tucker_interaction
 
+    # default core tensor initialization
+    # cf. https://github.com/ibalazevic/TuckER/blob/master/model.py#L12
+    default_core_initializer: ClassVar[Initializer] = staticmethod(nn.init.uniform_)  # type: ignore
+    default_core_initializer_kwargs: Mapping[str, Any] = {"a": -1.0, "b": 1.0}
+
     def __init__(
         self,
         embedding_dim: int = 200,
@@ -1013,6 +993,8 @@ class TuckerInteraction(FunctionalInteraction[FloatTensor, FloatTensor, FloatTen
         relation_dropout: float = 0.4,
         head_relation_dropout: float = 0.5,
         apply_batch_normalization: bool = True,
+        core_initializer: Hint[Initializer] = None,
+        core_initializer_kwargs: OptionalKwargs = None,
     ):
         """Initialize the Tucker interaction function.
 
@@ -1028,9 +1010,22 @@ class TuckerInteraction(FunctionalInteraction[FloatTensor, FloatTensor, FloatTen
             The dropout rate applied to the combined head and relation representations.
         :param apply_batch_normalization:
             Whether to use batch normalization on head representations and the combination of head and relation.
+        :param core_initializer:
+            the core tensor's initializer, or a hint thereof
+        :param core_initializer_kwargs:
+            additional keyword-based parameters for the initializer
         """
         super().__init__()
 
+        # normalize initializer
+        if core_initializer is None:
+            core_initializer = self.default_core_initializer
+        self.core_initializer = core_initializer
+        if core_initializer is self.default_core_initializer and core_initializer_kwargs is None:
+            core_initializer_kwargs = self.default_core_initializer_kwargs
+        self.core_initializer_kwargs = core_initializer_kwargs
+
+        # normalize relation dimension
         if relation_dim is None:
             relation_dim = embedding_dim
 
@@ -1052,9 +1047,12 @@ class TuckerInteraction(FunctionalInteraction[FloatTensor, FloatTensor, FloatTen
         else:
             self.head_batch_norm = self.head_relation_batch_norm = None
 
+        self.reset_parameters()
+
     def reset_parameters(self):  # noqa:D102
-        # Initialize core tensor, cf. https://github.com/ibalazevic/TuckER/blob/master/model.py#L12
-        nn.init.uniform_(self.core_tensor, -1.0, 1.0)
+        # instantiate here to make module easily serializable
+        core_initializer = initializer_resolver.make(self.core_initializer, pos_kwargs=self.core_initializer_kwargs)
+        core_initializer(self.core_tensor)
         # batch norm gets reset automatically, since it defines reset_parameters
 
     def _prepare_state_for_functional(self) -> MutableMapping[str, Any]:
@@ -1407,15 +1405,18 @@ class MonotonicAffineTransformationInteraction(
         # forward entity/relation shapes
         self.entity_shape = base.entity_shape
         self.relation_shape = base.relation_shape
-        self.tail_entity_shape = base.tail_entity_shape
+        self._tail_entity_shape = base._tail_entity_shape
 
         # The parameters of the affine transformation: bias
         self.bias = nn.Parameter(torch.empty(size=tuple()), requires_grad=trainable_bias)
-        self.initial_bias = torch.as_tensor(data=[initial_bias], dtype=torch.get_default_dtype())
+        self.initial_bias = torch.as_tensor(data=[initial_bias], dtype=torch.get_default_dtype()).squeeze()
 
         # scale. We model this as log(scale) to ensure scale > 0, and thus monotonicity
         self.log_scale = nn.Parameter(torch.empty(size=tuple()), requires_grad=trainable_scale)
-        self.initial_log_scale = torch.as_tensor(data=[math.log(initial_scale)], dtype=torch.get_default_dtype())
+        self.initial_log_scale = torch.as_tensor(
+            data=[math.log(initial_scale)],
+            dtype=torch.get_default_dtype(),
+        ).squeeze()
 
     def reset_parameters(self):  # noqa: D102
         self.bias.data = self.initial_bias.to(device=self.bias.device)
@@ -1464,7 +1465,7 @@ class CrossEInteraction(FunctionalInteraction[FloatTensor, Tuple[FloatTensor, Fl
             combination_activation,
             pos_kwargs=combination_activation_kwargs,
         )
-        self.combination_bias = nn.Parameter(data=torch.zeros(1, 1, 1, 1, embedding_dim))
+        self.combination_bias = nn.Parameter(data=torch.zeros(embedding_dim))
         self.combination_dropout = nn.Dropout(combination_dropout) if combination_dropout else None
 
     def _prepare_state_for_functional(self) -> MutableMapping[str, Any]:  # noqa: D102
@@ -1549,7 +1550,7 @@ class CPInteraction(FunctionalInteraction[FloatTensor, FloatTensor, FloatTensor]
     An implementation of the CP interaction as described [lacroix2018]_ (originally from [hitchcock1927]_).
 
     .. note ::
-        For $k=1$, this interaction is the same as DistMult (but consider the node below).
+        For $k=1$, this interaction is the same as DistMult (but consider the note below).
 
     .. note ::
         For equivalence to CP, entities should have different representations for head & tail role. This is different
@@ -1559,6 +1560,78 @@ class CPInteraction(FunctionalInteraction[FloatTensor, FloatTensor, FloatTensor]
     func = pkf.cp_interaction
     entity_shape = ("kd",)
     relation_shape = ("kd",)
+    _head_indices = (0,)
+    _tail_indices = (1,)
+
+
+@parse_docdata
+class MultiLinearTuckerInteraction(
+    FunctionalInteraction[Tuple[FloatTensor, FloatTensor], FloatTensor, Tuple[FloatTensor, FloatTensor]]
+):
+    """
+    An implementation of the original (multi-linear) TuckER interaction as described [tucker1966]_.
+
+    .. note ::
+        For small tensors, there are more efficient algorithms to compute the decomposition, e.g.,
+        http://tensorly.org/stable/modules/generated/tensorly.decomposition.Tucker.html
+
+    ---
+    name: MultiLinearTucker
+    citation:
+        author: Tucker
+        year: 1966
+        link: https://dx.doi.org/10.1007/BF02289464
+    """
+
+    func = pkf.multilinear_tucker_interaction
+    entity_shape = ("d", "f")
+    relation_shape = ("e",)
+
+    def __init__(
+        self,
+        head_dim: int = 64,
+        relation_dim: Optional[int] = None,
+        tail_dim: Optional[int] = None,
+    ):
+        """Initialize the Tucker interaction function.
+
+        :param head_dim:
+            The head entity embedding dimension.
+        :param relation_dim:
+            The relation embedding dimension. Defaults to `head_dim`.
+        :param tail_dim:
+            The tail entity embedding dimension. Defaults to `head_dim`.
+        """
+        super().__init__()
+
+        # input normalization
+        relation_dim = relation_dim or head_dim
+        tail_dim = tail_dim or head_dim
+
+        # Core tensor
+        self.core_tensor = nn.Parameter(
+            torch.empty(head_dim, relation_dim, tail_dim),
+            requires_grad=True,
+        )
+
+    def reset_parameters(self):  # noqa:D102
+        # initialize core tensor
+        nn.init.normal_(
+            self.core_tensor,
+            mean=0,
+            std=numpy.sqrt(numpy.prod(numpy.reciprocal(numpy.asarray(self.core_tensor.shape)))),
+        )
+
+    @staticmethod
+    def _prepare_hrt_for_functional(
+        h: Tuple[FloatTensor, FloatTensor],
+        r: FloatTensor,
+        t: Tuple[FloatTensor, FloatTensor],
+    ) -> MutableMapping[str, torch.FloatTensor]:
+        return dict(h=h[0], r=r, t=t[1])
+
+    def _prepare_state_for_functional(self) -> MutableMapping[str, Any]:
+        return dict(core_tensor=self.core_tensor)
 
 
 @parse_docdata
@@ -1631,7 +1704,7 @@ class TripleREInteraction(
         FloatTensor,
     ]
 ):
-    """A stateful module for the TripleRE interaction function.
+    """A stateful module for the TripleRE interaction function from [yu2021]_.
 
     .. seealso:: :func:`pykeen.nn.functional.triple_re_interaction`
 
@@ -1655,8 +1728,11 @@ class TripleREInteraction(
 
         :param u:
             the relation factor offset. can be set to None to disable it.
-        :param kwargs:
-            additional keyword-based arguments passed to :class:`NormBasedInteraction`
+        :param p:
+            The norm used with :func:`torch.linalg.vector_norm`. Defaults to 1 for TripleRE.
+        :param power_norm:
+            Whether to use the p-th power of the $L_p$ norm. It has the advantage of being differentiable around 0,
+            and numerically more stable. Defaults to False for TripleRE.
         """
         super().__init__(p=p, power_norm=power_norm)
         self.u = u
@@ -1699,7 +1775,7 @@ class AutoSFInteraction(FunctionalInteraction[HeadRepresentation, RelationRepres
                 3. tail_representation_index,
                 4. sign
 
-        :raise ValueError:
+        :raises ValueError:
             if there are duplicate coefficients
         """
         super().__init__()
@@ -1725,8 +1801,11 @@ class AutoSFInteraction(FunctionalInteraction[HeadRepresentation, RelationRepres
 
         :param coefficients:
             the coefficients in the "official" serialization format.
+        :returns:
+            An AutoSF interaction module
 
-        cf. https://github.com/AutoML-Research/AutoSF/blob/07b7243ccf15e579176943c47d6e65392cd57af3/searched_SFs.txt
+        .. seealso::
+            https://github.com/AutoML-Research/AutoSF/blob/07b7243ccf15e579176943c47d6e65392cd57af3/searched_SFs.txt
         """
         return cls(
             coefficients=[(i, ri, i, 1) for i, ri in enumerate(coefficients[:4])]
@@ -1766,7 +1845,7 @@ class AutoSFInteraction(FunctionalInteraction[HeadRepresentation, RelationRepres
         )
 
 
-interaction_resolver = Resolver.from_subclasses(
+interaction_resolver: ClassResolver[Interaction] = ClassResolver.from_subclasses(
     Interaction,  # type: ignore
     skip={NormBasedInteraction, FunctionalInteraction, MonotonicAffineTransformationInteraction},
     suffix=Interaction.__name__,
