@@ -37,7 +37,7 @@ could be used as in:
 >>> pipeline_result.save_to_directory('nations_transe')
 
 In this example, the dataset was given as a string. A list of available datasets can be found in
-:mod:`pykeen.datasets`. Alternatively, a subclass of :class:`pykeen.datasets.base.Dataset` could be
+:mod:`pykeen.datasets`. Alternatively, a subclass of :class:`pykeen.datasets.Dataset` could be
 used as in:
 
 >>> from pykeen.pipeline import pipeline
@@ -197,13 +197,14 @@ from typing import Any, Collection, Dict, Iterable, List, Mapping, MutableMappin
 import pandas as pd
 import scipy.stats
 import torch
+from class_resolver.utils import OneOrManyHintOrType, OneOrManyOptionalKwargs
 from torch.optim.optimizer import Optimizer
 
 from ..constants import PYKEEN_CHECKPOINTS, USER_DEFINED_CODE
 from ..datasets import get_dataset
 from ..datasets.base import Dataset
 from ..evaluation import Evaluator, MetricResults, evaluator_resolver
-from ..evaluation.rank_based_evaluator import resolve_metric_name
+from ..evaluation.ranking_metric_lookup import normalize_flattened_metric_results
 from ..losses import Loss, loss_resolver
 from ..lr_schedulers import LRScheduler, lr_scheduler_resolver
 from ..models import Model, make_model_cls, model_resolver
@@ -215,12 +216,11 @@ from ..stoppers import EarlyStopper, Stopper, stopper_resolver
 from ..trackers import ResultTracker, resolve_result_trackers
 from ..training import SLCWATrainingLoop, TrainingLoop, training_loop_resolver
 from ..triples import CoreTriplesFactory
-from ..typing import Hint, HintType, MappedTriples, OneOrSequence
+from ..typing import Hint, HintType, MappedTriples
 from ..utils import (
     Result,
     ensure_ftp_directory,
     fix_dataclass_init_docs,
-    flatten_dictionary,
     get_json_bytes_io,
     get_model_io,
     load_configuration,
@@ -376,6 +376,7 @@ class PipelineResult(Result):
         *,
         save_metadata: bool = True,
         save_replicates: bool = True,
+        save_training: bool = True,
         **_kwargs,
     ) -> None:
         """Save all artifacts in the given directory."""
@@ -383,12 +384,16 @@ class PipelineResult(Result):
             directory = pathlib.Path(directory).resolve()
         directory.mkdir(exist_ok=True, parents=True)
 
-        with directory.joinpath("metadata.json").open("w") as file:
-            json.dump(self.metadata, file, indent=2, sort_keys=True)
         with directory.joinpath("results.json").open("w") as file:
             json.dump(self._get_results(), file, indent=2, sort_keys=True)
+        if save_metadata:
+            with directory.joinpath("metadata.json").open("w") as file:
+                json.dump(self.metadata, file, indent=2, sort_keys=True)
         if save_replicates:
             self.save_model(directory.joinpath("trained_model.pkl"))
+        if save_training:
+            self.training.to_path_binary(directory.joinpath("training_triples"))
+        logger.info(f"Saved to directory: {directory.as_uri()}")
 
     def save_to_ftp(self, directory: str, ftp: ftplib.FTP) -> None:
         """Save all artifacts to the given directory in the FTP server.
@@ -501,6 +506,8 @@ def replicate_pipeline_from_config(
     replicates: int,
     move_to_cpu: bool = False,
     save_replicates: bool = True,
+    save_training: bool = False,
+    keep_seed: bool = False,
     **kwargs,
 ) -> None:
     """Run the same pipeline several times from a configuration dictionary.
@@ -511,21 +518,27 @@ def replicate_pipeline_from_config(
     :param move_to_cpu: Should the models be moved back to the CPU? Only relevant if training on GPU.
     :param save_replicates: Should the artifacts of the replicates be saved?
     :param kwargs: Keyword arguments to be passed through to :func:`pipeline_from_config`.
+    :param save_training: whether to save the training factory.
+    :param keep_seed: whether to keep the random seed in a configuration
     """
-    pipeline_results = (pipeline_from_config(config, **kwargs) for _ in range(replicates))
+    # note: we do not directly forward discard_seed here, since we want to highlight the different default behaviour:
+    #    when replicating (i.e., running multiple replicates), fixing a random seed would render the replicates useless
+    pipeline_results = (pipeline_from_config(config, discard_seed=not keep_seed, **kwargs) for _ in range(replicates))
     save_pipeline_results_to_directory(
         config=config,
         directory=directory,
         pipeline_results=pipeline_results,
         move_to_cpu=move_to_cpu,
         save_replicates=save_replicates,
+        save_training=save_training,
     )
 
 
 def _iterate_moved(pipeline_results: Iterable[PipelineResult]):
     for pipeline_result in pipeline_results:
-        pipeline_result.model.device = resolve_device("cpu")
-        pipeline_result.model.to_device_()
+        # note: torch.nn.Module.cpu() is in-place in contrast to torch.Tensor.cpu()
+        pipeline_result.model.cpu()
+        torch.cuda.empty_cache()
         yield pipeline_result
 
 
@@ -542,9 +555,7 @@ class _ResultAccumulator:
 
     def add_original_result(self, result: Mapping[str, Any]) -> None:
         """Add an "original" result, i.e., one stored in the reproducibility configuration."""
-        # normalize keys
-        # TODO: this can only normalize rank-based metrics!
-        result = {str(resolve_metric_name(k)): v for k, v in flatten_dictionary(result).items()}
+        result = normalize_flattened_metric_results(result)
         self.keys = sorted(result.keys())
         self.data.append([True] + [result[k] for k in self.keys])
 
@@ -620,6 +631,7 @@ def save_pipeline_results_to_directory(
     move_to_cpu: bool = False,
     save_metadata: bool = False,
     save_replicates: bool = True,
+    save_training: bool = False,
     width: int = 5,
 ) -> None:
     """Save the result set to the directory.
@@ -631,6 +643,7 @@ def save_pipeline_results_to_directory(
     :param save_metadata: Should the metadata be saved? Might be redundant in a scenario when you're
         using this function, so defaults to false.
     :param save_replicates: Should the artifacts of the replicates be saved?
+    :param save_training: Should the training triples be saved?
     :param width: How many leading zeros should be put in the replicate names?
     """
     if isinstance(directory, str):
@@ -652,6 +665,7 @@ def save_pipeline_results_to_directory(
             replicate_directory,
             save_metadata=save_metadata,
             save_replicates=save_replicates,
+            save_training=save_training,
         )
         for epoch, loss in enumerate(pipeline_result.losses):
             losses_rows.append((i, epoch, loss))
@@ -691,6 +705,7 @@ def pipeline_from_path(
 
 def pipeline_from_config(
     config: Mapping[str, Any],
+    discard_seed: bool = False,
     **kwargs,
 ) -> PipelineResult:
     """Run the pipeline with a configuration dictionary.
@@ -698,6 +713,8 @@ def pipeline_from_config(
     :param config: The experiment configuration dictionary. Should have a 'metadata' and 'pipeline'
         key. The metadata entry is passed to the metadata argument of :func:`pipeline`. The 'pipeline'
         entry is passed via splat to :func:`pipeline`.
+    :param discard_seed:
+        whether to discard the random seed for the pipeline, if present.
     :param kwargs: Additional kwargs to forward to :func:`pipeline`.
     :return: The results of running the pipeline on the given configuration.
     """
@@ -705,6 +722,9 @@ def pipeline_from_config(
     title = metadata.get("title")
     if title is not None:
         logger.info(f"Running: {title}")
+    if discard_seed and "random_seed" in pipeline_kwargs:
+        random_seed = pipeline_kwargs.pop("random_seed")
+        logger.info(f"Ignoring random_seed={random_seed}. You need to explicitly disable this, if unintended.")
 
     return pipeline(
         metadata=metadata,
@@ -741,7 +761,6 @@ def _build_model_helper(
     if model_kwargs is None:
         model_kwargs = {}
     model_kwargs = dict(model_kwargs)
-    model_kwargs.update(preferred_device=_device)
     model_kwargs.setdefault("random_seed", _random_seed)
 
     if regularizer is not None:
@@ -822,8 +841,8 @@ def pipeline(  # noqa: C901
     evaluator_kwargs: Optional[Mapping[str, Any]] = None,
     evaluation_kwargs: Optional[Mapping[str, Any]] = None,
     # 9. Tracking
-    result_tracker: Optional[OneOrSequence[HintType[ResultTracker]]] = None,
-    result_tracker_kwargs: Optional[OneOrSequence[Optional[Mapping[str, Any]]]] = None,
+    result_tracker: OneOrManyHintOrType[ResultTracker] = None,
+    result_tracker_kwargs: OneOrManyOptionalKwargs = None,
     # Misc
     metadata: Optional[Dict[str, Any]] = None,
     device: Hint[torch.device] = None,
@@ -999,6 +1018,7 @@ def pipeline(  # noqa: C901
     _result_tracker.start_run(run_name=title)
 
     _device: torch.device = resolve_device(device)
+    logger.info(f"Using device: {device}")
 
     dataset_instance: Dataset = get_dataset(
         dataset=dataset,
@@ -1068,6 +1088,8 @@ def pipeline(  # noqa: C901
             training_triples_factory=training,
         )
 
+    model_instance = model_instance.to(_device)
+
     # Log model parameters
     _result_tracker.log_params(
         params=dict(
@@ -1076,6 +1098,20 @@ def pipeline(  # noqa: C901
         ),
     )
 
+    # Log loss parameters
+    _result_tracker.log_params(
+        params=dict(
+            # the loss was already logged as part of the model kwargs
+            # loss=loss_resolver.normalize_inst(model_instance.loss),
+            loss_kwargs=loss_kwargs
+        ),
+    )
+
+    # Log regularizer parameters
+    _result_tracker.log_params(
+        params=dict(regularizer_kwargs=regularizer_kwargs),
+    )
+
     optimizer_kwargs = dict(optimizer_kwargs or {})
     optimizer_instance = optimizer_resolver.make(
         optimizer,
@@ -1133,6 +1169,7 @@ def pipeline(  # noqa: C901
         triples_factory=training,
         optimizer=optimizer_instance,
         lr_scheduler=lr_scheduler_instance,
+        result_tracker=_result_tracker,
         **training_loop_kwargs,
     )
     _result_tracker.log_params(
@@ -1200,7 +1237,6 @@ def pipeline(  # noqa: C901
     losses = training_loop_instance.train(
         triples_factory=training,
         stopper=stopper_instance,
-        result_tracker=_result_tracker,
         clear_optimizer=clear_optimizer,
         **training_kwargs,
     )
