@@ -26,9 +26,7 @@ The following code will create a scenario in which training will stop
 ... )
 """
 
-from typing import Collection, Type
-
-from class_resolver import Resolver, get_subclasses
+from class_resolver import ClassResolver
 
 from .early_stopping import EarlyStopper, StopperCallback  # noqa: F401
 from .stopper import NopStopper, Stopper
@@ -41,11 +39,7 @@ __all__ = [
     "stopper_resolver",
 ]
 
-_STOPPER_SUFFIX = "Stopper"
-_STOPPERS: Collection[Type[Stopper]] = set(get_subclasses(Stopper))  # type: ignore
-stopper_resolver = Resolver(
-    _STOPPERS,
+stopper_resolver = ClassResolver.from_subclasses(
+    Stopper,
     default=NopStopper,
-    suffix=_STOPPER_SUFFIX,
-    base=Stopper,  # type: ignore
 )
