@@ -2,23 +2,13 @@
 
 """Classes for creating and storing training data from triples."""
 
-from .instances import (
-    Instances,
-    LCWAInstances,
-    MultimodalInstances,
-    MultimodalLCWAInstances,
-    MultimodalSLCWAInstances,
-    SLCWAInstances,
-)
+from .instances import Instances, LCWAInstances, SLCWAInstances
 from .triples_factory import CoreTriplesFactory, RelationInverter, TriplesFactory, relation_inverter
 from .triples_numeric_literals_factory import TriplesNumericLiteralsFactory
 
 __all__ = [
     "Instances",
     "LCWAInstances",
-    "MultimodalInstances",
-    "MultimodalSLCWAInstances",
-    "MultimodalLCWAInstances",
     "SLCWAInstances",
     "CoreTriplesFactory",
     "RelationInverter",
