@@ -13,14 +13,14 @@ import pandas as pd
 from tqdm.auto import tqdm
 
 from . import TriplesFactory
-from ..typing import MappedTriples
+from ..constants import TARGET_TO_INDEX
+from ..typing import COLUMN_HEAD, COLUMN_RELATION, COLUMN_TAIL, LABEL_HEAD, LABEL_RELATION, LABEL_TAIL, MappedTriples
 
 logger = logging.getLogger(__name__)
 
 __all__ = [
     "add_entity_labels",
     "add_relation_labels",
-    "create_relation_to_entity_set_mapping",
     "entity_relation_co_occurrence",
     "get_entity_counts",
     "get_relation_counts",
@@ -72,8 +72,6 @@ CARDINALITY_TYPE_COLUMN_NAME = "relation_type"
 PATTERN_TYPE_COLUMN_NAME = "pattern"
 CONFIDENCE_COLUMN_NAME = "confidence"
 SUPPORT_COLUMN_NAME = "support"
-POSITION_TAIL = "tail"
-POSITION_HEAD = "head"
 
 
 def _add_labels(
@@ -193,26 +191,6 @@ def index_relations(
     return dict(ins), dict(outs)
 
 
-def create_relation_to_entity_set_mapping(
-    triples: Iterable[Tuple[int, int, int]],
-) -> Tuple[Mapping[int, Set[int]], Mapping[int, Set[int]]]:
-    """
-    Create mappings from relation IDs to the set of their head / tail entities.
-
-    :param triples:
-        The triples.
-
-    :return:
-        A pair of dictionaries, each mapping relation IDs to entity ID sets.
-    """
-    tails = defaultdict(set)
-    heads = defaultdict(set)
-    for h, r, t in triples:
-        heads[r].add(h)
-        tails[r].add(t)
-    return heads, tails
-
-
 def index_pairs(triples: Iterable[Tuple[int, int, int]]) -> Mapping[int, Set[Tuple[int, int]]]:
     """Create a mapping from relation to head/tail pairs for fast lookup."""
     rv: DefaultDict[int, Set[Tuple[int, int]]] = defaultdict(set)
@@ -426,10 +404,10 @@ def iter_relation_cardinality_types(
 def _help_iter_relation_cardinality_types(
     mapped_triples: Collection[Tuple[int, int, int]],
 ) -> Iterable[Tuple[int, int, float, float]]:
-    df = pd.DataFrame(data=mapped_triples, columns=["h", "r", "t"])
-    for relation, group in df.groupby(by="r"):
-        n_unique_heads, head_injective_conf = _is_injective_mapping(df=group, source="h", target="t")
-        n_unique_tails, tail_injective_conf = _is_injective_mapping(df=group, source="t", target="h")
+    df = pd.DataFrame(data=mapped_triples, columns=[LABEL_HEAD, LABEL_RELATION, LABEL_TAIL])
+    for relation, group in df.groupby(by=LABEL_RELATION):
+        n_unique_heads, head_injective_conf = _is_injective_mapping(df=group, source=LABEL_HEAD, target=LABEL_TAIL)
+        n_unique_tails, tail_injective_conf = _is_injective_mapping(df=group, source=LABEL_TAIL, target=LABEL_HEAD)
         # TODO: what is the support?
         support = n_unique_heads + n_unique_tails
         yield relation, support, head_injective_conf, tail_injective_conf
@@ -492,10 +470,8 @@ def get_entity_counts(
         A dataframe with columns ( entity_id | type | count )
     """
     data = []
-    for label, col in (
-        (POSITION_HEAD, 0),
-        (POSITION_TAIL, 2),
-    ):
+    for label in (LABEL_HEAD, LABEL_TAIL):
+        col = TARGET_TO_INDEX[label]
         unique, counts = _get_counts(mapped_triples=mapped_triples, column=col)
         df = pd.DataFrame(
             {
@@ -593,7 +569,7 @@ def relation_injectivity(
     it = _help_iter_relation_cardinality_types(mapped_triples)
     df = pd.DataFrame(
         data=it,
-        columns=[RELATION_ID_COLUMN_NAME, SUPPORT_COLUMN_NAME, POSITION_HEAD, POSITION_TAIL],
+        columns=[RELATION_ID_COLUMN_NAME, SUPPORT_COLUMN_NAME, LABEL_HEAD, LABEL_TAIL],
     )
     return add_relation_labels(df, add_labels=add_labels, label_to_id=label_to_id)
 
@@ -664,10 +640,10 @@ def entity_relation_co_occurrence(
     """
     data = []
 
-    for name, columns in dict(
-        head=[0, 1],
-        tail=[2, 1],
-    ).items():
+    for name, columns in {
+        LABEL_HEAD: (COLUMN_HEAD, COLUMN_RELATION),
+        LABEL_TAIL: (COLUMN_TAIL, COLUMN_RELATION),
+    }.items():
         unique, counts = mapped_triples[:, columns].unique(dim=0, return_counts=True)
         e, r = unique.t().numpy()
         df = pd.DataFrame(
@@ -696,15 +672,10 @@ def get_relation_functionality(
     :return:
         A dataframe with columns ( functionality | inverse_functionality )
     """
-    df = pd.DataFrame(data=mapped_triples, columns=["h", "r", "t"])
-    df = df.groupby(by="r").agg(
-        dict(
-            h=["nunique", COUNT_COLUMN_NAME],
-            t="nunique",
-        )
-    )
-    df[FUNCTIONALITY_COLUMN_NAME] = df[("h", "nunique")] / df[("h", COUNT_COLUMN_NAME)]
-    df[INVERSE_FUNCTIONALITY_COLUMN_NAME] = df[("t", "nunique")] / df[("h", COUNT_COLUMN_NAME)]
+    df = pd.DataFrame(data=mapped_triples, columns=[LABEL_HEAD, LABEL_RELATION, LABEL_TAIL])
+    df = df.groupby(by=LABEL_RELATION).agg({LABEL_HEAD: ["nunique", COUNT_COLUMN_NAME], LABEL_TAIL: "nunique"})
+    df[FUNCTIONALITY_COLUMN_NAME] = df[(LABEL_HEAD, "nunique")] / df[(LABEL_HEAD, COUNT_COLUMN_NAME)]
+    df[INVERSE_FUNCTIONALITY_COLUMN_NAME] = df[(LABEL_TAIL, "nunique")] / df[(LABEL_HEAD, COUNT_COLUMN_NAME)]
     df = df[[FUNCTIONALITY_COLUMN_NAME, INVERSE_FUNCTIONALITY_COLUMN_NAME]]
     df.columns = df.columns.droplevel(1)
     df.index.name = RELATION_ID_COLUMN_NAME
