@@ -4,7 +4,6 @@
 
 import ftplib
 import functools
-import inspect
 import itertools as itt
 import json
 import logging
@@ -13,12 +12,15 @@ import operator
 import os
 import pathlib
 import random
+import re
 from abc import ABC, abstractmethod
+from collections import defaultdict
 from io import BytesIO
 from pathlib import Path
 from typing import (
     Any,
     Callable,
+    Collection,
     Dict,
     Generic,
     Iterable,
@@ -26,6 +28,7 @@ from typing import (
     Mapping,
     Optional,
     Sequence,
+    Set,
     Tuple,
     Type,
     TypeVar,
@@ -38,9 +41,9 @@ import torch
 import torch.nn
 import torch.nn.modules.batchnorm
 import yaml
-from class_resolver import Resolver, normalize_string
+from class_resolver import normalize_string
+from docdata import get_docdata
 from torch import nn
-from torch.nn import functional
 
 from .constants import PYKEEN_BENCHMARKS
 from .typing import DeviceHint, MappedTriples, TorchRandomHint
@@ -48,9 +51,11 @@ from .version import get_git_hash
 
 __all__ = [
     "at_least_eps",
+    "broadcast_upgrade_to_sequences",
     "compose",
     "clamp_norm",
     "compact_mapping",
+    "create_relation_to_entity_set_mapping",
     "ensure_torch_random_state",
     "format_relative_comparison",
     "invert_mapping",
@@ -68,7 +73,6 @@ __all__ = [
     "fix_dataclass_init_docs",
     "get_benchmark",
     "extended_einsum",
-    "strip_dim",
     "upgrade_to_sequence",
     "ensure_tuple",
     "unpack_singletons",
@@ -82,7 +86,6 @@ __all__ = [
     "get_json_bytes_io",
     "get_df_io",
     "ensure_ftp_directory",
-    "broadcast_cat",
     "get_batchnorm_modules",
     "get_dropout_modules",
     "calculate_broadcasted_elementwise_result_shape",
@@ -97,13 +100,17 @@ __all__ = [
     "convert_to_canonical_shape",
     "get_expected_norm",
     "Bias",
-    "activation_resolver",
     "complex_normalize",
     "lp_norm",
     "powersum_norm",
     "product_normalize",
     "compute_box",
     "point_to_box_distance",
+    "get_devices",
+    "get_preferred_device",
+    "triple_tensor_to_set",
+    "is_triple_tensor_subset",
+    "logcumsumexp",
 ]
 
 logger = logging.getLogger(__name__)
@@ -136,6 +143,43 @@ def resolve_device(device: DeviceHint = None) -> torch.device:
     return device
 
 
+class DeviceResolutionError(ValueError):
+    """An error in the resolution of a model's device."""
+
+
+class AmbiguousDeviceError(DeviceResolutionError):
+    """An error raised if there is ambiguity in device resolution."""
+
+    def __init__(self, module: nn.Module) -> None:
+        """Initialize the error."""
+        _info = defaultdict(list)
+        for name, tensor in itt.chain(module.named_parameters(), module.named_buffers()):
+            _info[tensor.data.device].append(name)
+        info = {device: sorted(tensor_names) for device, tensor_names in _info.items()}
+        super().__init__(f"Ambiguous device! Found: {list(info.keys())}\n\n{info}")
+
+
+def get_devices(module: nn.Module) -> Collection[torch.device]:
+    """Return the device(s) from each components of the model."""
+    return {tensor.data.device for tensor in itt.chain(module.parameters(), module.buffers())}
+
+
+def get_preferred_device(module: nn.Module, allow_ambiguity: bool = True) -> torch.device:
+    """Return the preferred device."""
+    devices = get_devices(module=module)
+    if len(devices) == 0:
+        raise DeviceResolutionError("Could not infer device, since there are neither parameters nor buffers.")
+    if len(devices) == 1:
+        return next(iter(devices))
+    if not allow_ambiguity:
+        raise AmbiguousDeviceError(module=module)
+    # try to resolve ambiguous device; there has to be at least one cuda device
+    cuda_devices = {d for d in devices if d.type == "cuda"}
+    if len(cuda_devices) == 1:
+        return next(iter(cuda_devices))
+    raise AmbiguousDeviceError(module=module)
+
+
 X = TypeVar("X")
 
 
@@ -209,12 +253,19 @@ def clamp_norm(
 class compose(Generic[X]):  # noqa:N801
     """A class representing the composition of several functions."""
 
-    def __init__(self, *operations: Callable[[X], X]):
+    def __init__(self, *operations: Callable[[X], X], name: str):
         """Initialize the composition with a sequence of operations.
 
         :param operations: unary operations that will be applied in succession
+        :param name: The name of the composed function.
         """
         self.operations = operations
+        self.name = name
+
+    @property
+    def __name__(self) -> str:
+        """Get the name of this composition."""
+        return self.name
 
     def __call__(self, x: X) -> X:
         """Apply the operations in order to the given tensor."""
@@ -333,8 +384,8 @@ def split_complex(
     x: torch.FloatTensor,
 ) -> Tuple[torch.FloatTensor, torch.FloatTensor]:
     """Split a complex tensor into real and imaginary part."""
-    dim = x.shape[-1] // 2
-    return x[..., :dim], x[..., dim:]
+    x = torch.view_as_real(x)
+    return x[..., 0], x[..., 1]
 
 
 def view_complex(x: torch.FloatTensor) -> torch.Tensor:
@@ -353,7 +404,7 @@ def combine_complex(
     x_im: torch.FloatTensor,
 ) -> torch.FloatTensor:
     """Combine a complex tensor from real and imaginary part."""
-    return torch.cat([x_re, x_im], dim=-1)
+    return torch.view_as_complex(torch.stack([x_re, x_im], dim=-1))
 
 
 def fix_dataclass_init_docs(cls: Type) -> Type:
@@ -455,63 +506,6 @@ def format_relative_comparison(
     return f"{part}/{total} ({part / total:2.2%})"
 
 
-def broadcast_cat(
-    tensors: Sequence[torch.FloatTensor],
-    dim: int,
-) -> torch.FloatTensor:
-    """Concatenate tensors with broadcasting support.
-
-    :param tensors:
-        The tensors. Each of the tensors is require to have the same number of dimensions.
-        For each dimension not equal to dim, the extent has to match the other tensors', or be one.
-        If it is one, the tensor is repeated to match the extent of the othe tensors.
-    :param dim:
-        The concat dimension.
-
-    :return: A concatenated, broadcasted tensor.
-
-    :raises ValueError: if the x and y dimensions are not the same
-    :raises ValueError: if broadcasting is not possible
-    """
-    # input validation
-    if len(tensors) == 0:
-        raise ValueError("Must pass at least one tensor.")
-    if len({x.ndimension() for x in tensors}) != 1:
-        raise ValueError(
-            f"The number of dimensions has to be the same for all tensors, but is {set(t.shape for t in tensors)}",
-        )
-
-    # base case
-    if len(tensors) == 1:
-        return tensors[0]
-
-    # normalize dim
-    if dim < 0:
-        dim = tensors[0].ndimension() + dim
-
-    # calculate repeats for each tensor
-    repeats = [[1 for _ in t.shape] for t in tensors]
-    for i, dims in enumerate(zip(*(t.shape for t in tensors))):
-        # dimensions along concatenation axis do not need to match
-        if i == dim:
-            continue
-
-        # get desired extent along dimension
-        d_max = max(dims)
-        if not {1, d_max}.issuperset(dims):
-            raise ValueError(f"Tensors have invalid shape along {i} dimension: {set(dims)}")
-
-        for j, td in enumerate(dims):
-            if td != d_max:
-                repeats[j][i] = d_max
-
-    # repeat tensors along axes if necessary
-    tensors = [t.repeat(*r) for t, r in zip(tensors, repeats)]
-
-    # concatenate
-    return torch.cat(tensors, dim=dim)
-
-
 def get_batchnorm_modules(module: torch.nn.Module) -> List[torch.nn.Module]:
     """Return all submodules which are batch normalization layers."""
     return [submodule for submodule in module.modules() if isinstance(submodule, torch.nn.modules.batchnorm._BatchNorm)]
@@ -607,7 +601,7 @@ def _reorder(
         return tensors
     # determine optimal processing order
     shapes = tuple(tuple(t.shape) for t in tensors)
-    if len(set(s[0] for s in shapes)) < 2:
+    if len(set(s[0] for s in shapes if s)) < 2:
         # heuristic
         return tensors
     order = get_optimal_sequence(*shapes)[1]
@@ -666,11 +660,6 @@ def negative_norm(
         assert not isinstance(p, str)
         return -(x.abs() ** p).sum(dim=-1)
 
-    if torch.is_complex(x):
-        assert not isinstance(p, str)
-        # workaround for complex numbers: manually compute norm
-        return -(x.abs() ** p).sum(dim=-1) ** (1 / p)
-
     return -x.norm(p=p, dim=-1)
 
 
@@ -750,9 +739,11 @@ def project_entity(
     return e_bot
 
 
+# TODO delete when deleting _normalize_dim (below)
 CANONICAL_DIMENSIONS = dict(h=1, r=2, t=3)
 
 
+# TODO delete when deleting convert_to_canonical_shape (below)
 def _normalize_dim(dim: Union[int, str]) -> int:
     """Normalize the dimension selection."""
     if isinstance(dim, int):
@@ -760,6 +751,7 @@ def _normalize_dim(dim: Union[int, str]) -> int:
     return CANONICAL_DIMENSIONS[dim.lower()[0]]
 
 
+# TODO delete? See note in test_sim.py on its only usage
 def convert_to_canonical_shape(
     x: torch.FloatTensor,
     dim: Union[int, str],
@@ -792,16 +784,6 @@ def convert_to_canonical_shape(
     return x.view(*shape, *suffix_shape)
 
 
-def strip_dim(*tensors: torch.FloatTensor, n: int = 4) -> Sequence[torch.FloatTensor]:
-    """Strip the first dimensions.
-
-    :param tensors: The tensors whose first ``n`` dimensions should be independently stripped
-    :param n: The number of initial dimensions to strip
-    :return: A tuple of the reduced tensors
-    """
-    return tuple(tensor.view(tensor.shape[n:]) for tensor in tensors)
-
-
 def upgrade_to_sequence(x: Union[X, Sequence[X]]) -> Sequence[X]:
     """Ensure that the input is a sequence.
 
@@ -835,6 +817,38 @@ def upgrade_to_sequence(x: Union[X, Sequence[X]]) -> Sequence[X]:
     return x if (isinstance(x, Sequence) and not isinstance(x, str)) else (x,)  # type: ignore
 
 
+def broadcast_upgrade_to_sequences(*xs: Union[X, Sequence[X]]) -> Sequence[Sequence[X]]:
+    """Apply upgrade_to_sequence to each input, and afterwards repeat singletons to match the maximum length.
+
+    :param xs: length: m
+        the inputs.
+
+    :return:
+        a sequence of length m, where each element is a sequence and all elements have the same length.
+
+    :raises ValueError:
+        if there is a non-singleton sequence input with length different from the maximum sequence length.
+
+    >>> broadcast_upgrade_to_sequences(1)
+    ((1,),)
+    >>> broadcast_upgrade_to_sequences(1, 2)
+    ((1,), (2,))
+    >>> broadcast_upgrade_to_sequences(1, (2, 3))
+    ((1, 1), (2, 3))
+    """
+    # upgrade to sequence
+    xs_ = [upgrade_to_sequence(x) for x in xs]
+    # broadcast
+    max_len = max(map(len, xs_))
+    for i in range(len(xs_)):
+        x = xs_[i]
+        if len(x) < max_len:
+            if len(x) != 1:
+                raise ValueError(f"Length mismatch: maximum length: {max_len}, but encountered length {len(x)}, too.")
+            xs_[i] = tuple(list(x) * max_len)
+    return tuple(xs_)
+
+
 def ensure_tuple(*x: Union[X, Sequence[X]]) -> Sequence[Sequence[X]]:
     """Ensure that all elements in the sequence are upgraded to sequences.
 
@@ -859,11 +873,6 @@ def unpack_singletons(*xs: Tuple[X]) -> Sequence[Union[X, Tuple[X]]]:
     return tuple(x[0] if len(x) == 1 else x for x in xs)
 
 
-def _can_slice(fn) -> bool:
-    """Check if a model's score_X function can slice."""
-    return "slice_size" in inspect.getfullargspec(fn).args
-
-
 def extend_batch(
     batch: MappedTriples,
     all_ids: List[int],
@@ -987,20 +996,6 @@ def get_expected_norm(
         raise TypeError(f"norm not implemented for {type(p)}: {p}")
 
 
-activation_resolver = Resolver(
-    classes=(
-        nn.LeakyReLU,
-        nn.PReLU,
-        nn.ReLU,
-        nn.Softplus,
-        nn.Sigmoid,
-        nn.Tanh,
-    ),
-    base=nn.Module,  # type: ignore
-    default=nn.ReLU,
-)
-
-
 class Bias(nn.Module):
     """A module wrapper for adding a bias."""
 
@@ -1048,29 +1043,23 @@ def powersum_norm(x: torch.FloatTensor, p: float, dim: Optional[int], normalize:
 
 
 def complex_normalize(x: torch.Tensor) -> torch.Tensor:
-    r"""Normalize a vector of complex numbers such that each element is of unit-length.
+    r"""Normalize a vector of complex numbers such that each *element* is of unit-length.
 
-    :param x: A tensor formulating complex numbers
-    :returns: A normalized version accoring to the following definition.
-
-    The `modulus of complex number <https://en.wikipedia.org/wiki/Absolute_value#Complex_numbers>`_ is given as:
+    Let $x \in \mathbb{C}^d$ denote a complex vector. Then, the operation computes
 
     .. math::
+        x_i' = \frac{x_i}{|x_i|}
 
-        |a + ib| = \sqrt{a^2 + b^2}
+    where $|x_i| = \sqrt{Re(x_i)^2 + Im(x_i)^2}$ is the
+    `modulus of complex number <https://en.wikipedia.org/wiki/Absolute_value#Complex_numbers>`_
 
-    $l_2$ norm of complex vector $x \in \mathbb{C}^d$:
+    :param x:
+        A tensor formulating complex numbers
 
-    .. math::
-        \|x\|^2 = \sum_{i=1}^d |x_i|^2
-                 = \sum_{i=1}^d \left(\operatorname{Re}(x_i)^2 + \operatorname{Im}(x_i)^2\right)
-                 = \left(\sum_{i=1}^d \operatorname{Re}(x_i)^2) + (\sum_{i=1}^d \operatorname{Im}(x_i)^2\right)
-                 = \|\operatorname{Re}(x)\|^2 + \|\operatorname{Im}(x)\|^2
-                 = \| [\operatorname{Re}(x); \operatorname{Im}(x)] \|^2
+    :returns:
+        An elementwise noramlized vector.
     """
-    y = x.view(*x.shape[:-1], x.shape[-1] // 2, 2)
-    y = functional.normalize(y, p=2, dim=-1)
-    return y.view(*x.shape)
+    return x / x.abs().clamp_min(torch.finfo(x.dtype).eps)
 
 
 CONFIGURATION_FILE_FORMATS = {".json", ".yaml", ".yml"}
@@ -1255,6 +1244,83 @@ def boxe_kg_arity_position_score(
     return negative_norm(element_wise_distance, p=p, power_norm=power_norm)
 
 
+def getattr_or_docdata(cls, key: str) -> str:
+    """Get the attr or data inside docdata."""
+    if hasattr(cls, key):
+        return getattr(cls, key)
+    getter_key = f"get_{key}"
+    if hasattr(cls, getter_key):
+        return getattr(cls, getter_key)()
+    docdata = get_docdata(cls)
+    if key in docdata:
+        return docdata[key]
+    raise KeyError
+
+
+def triple_tensor_to_set(tensor: torch.LongTensor) -> Set[Tuple[int, ...]]:
+    """Convert a tensor of triples to a set of int-tuples."""
+    return set(map(tuple, tensor.tolist()))
+
+
+def is_triple_tensor_subset(a: torch.LongTensor, b: torch.LongTensor) -> bool:
+    """Check whether one tensor of triples is a subset of another one."""
+    return triple_tensor_to_set(a).issubset(triple_tensor_to_set(b))
+
+
+def create_relation_to_entity_set_mapping(
+    triples: Iterable[Tuple[int, int, int]],
+) -> Tuple[Mapping[int, Set[int]], Mapping[int, Set[int]]]:
+    """
+    Create mappings from relation IDs to the set of their head / tail entities.
+
+    :param triples:
+        The triples.
+
+    :return:
+        A pair of dictionaries, each mapping relation IDs to entity ID sets.
+    """
+    tails = defaultdict(set)
+    heads = defaultdict(set)
+    for h, r, t in triples:
+        heads[r].add(h)
+        tails[r].add(t)
+    return heads, tails
+
+
+camel_to_snake_pattern = re.compile(r"(?<!^)(?=[A-Z])")
+
+
+def camel_to_snake(name: str) -> str:
+    """Convert camel-case to snake case."""
+    # cf. https://stackoverflow.com/a/1176023
+    return camel_to_snake_pattern.sub("_", name).lower()
+
+
+def make_ones_like(prefix: Sequence) -> Sequence[int]:
+    """Create a list of ones of same length as the input sequence."""
+    return [1 for _ in prefix]
+
+
+def logcumsumexp(a: np.ndarray) -> np.ndarray:
+    """Compute ``log(cumsum(exp(a)))``.
+
+    :param a: shape: s
+        the array
+
+    :return: shape s
+        the log-cumsum-exp of the array
+
+    .. seealso ::
+        :func:`scipy.special.logsumexp` and :func:`torch.logcumsumexp`
+    """
+    a_max = np.amax(a)
+    tmp = np.exp(a - a_max)
+    s = np.cumsum(tmp)
+    out = np.log(s)
+    out += a_max
+    return out
+
+
 if __name__ == "__main__":
     import doctest
 
