@@ -157,7 +157,7 @@ class TestPipeline(unittest.TestCase):
             ["head_id", "head_label", "relation_id", "relation_label", "tail_id", "tail_label", "score"],
             list(all_df.columns),
         )
-        possible = self.dataset.training.num_relations * self.model.num_entities ** 2
+        possible = self.dataset.training.num_relations * self.model.num_entities**2
         self.assertEqual(possible, len(all_df.index))
 
     def test_predict_all_remove_known(self):
@@ -173,7 +173,7 @@ class TestPipeline(unittest.TestCase):
             ["head_id", "head_label", "relation_id", "relation_label", "tail_id", "tail_label", "score"],
             list(all_df.columns),
         )
-        possible = self.dataset.training.num_relations * self.model.num_entities ** 2
+        possible = self.dataset.training.num_relations * self.model.num_entities**2
         known = self.dataset.training.num_triples + self.testing_mapped_triples.shape[0]
         self.assertNotEqual(possible, known, msg="testing and training triples cover all possible triples")
         self.assertEqual(possible - known, len(all_df.index))
@@ -200,7 +200,7 @@ class TestPipeline(unittest.TestCase):
             ],
             list(all_df.columns),
         )
-        possible = self.dataset.training.num_relations * self.model.num_entities ** 2
+        possible = self.dataset.training.num_relations * self.model.num_entities**2
         self.assertEqual(possible, len(all_df.index))
         self.assertEqual(self.dataset.training.num_triples, all_df["in_training"].sum())
         self.assertEqual(self.testing_mapped_triples.shape[0], all_df["in_testing"].sum())
@@ -364,8 +364,7 @@ class TestPipelineReplicate(unittest.TestCase):
                     model="transe",
                 ),
                 results={
-                    "hits_at_k": {"best": {"10": 0.538}},
-                    "mean_rank": {"best": 163},
+                    "best": {"hits_at_k": {"10": 0.538}, "mean_rank": 163},
                 },
             ),
             directory=self.tmp_dir_path,
@@ -516,7 +515,7 @@ class TestPipelineEvaluationFiltering(unittest.TestCase):
             random_seed=42,
             filter_validation_when_testing=False,
         )
-        assert results.metric_results.arithmetic_mean_rank["both"]["realistic"] == 2, "The rank should equal 2"
+        assert results.metric_results.get_metric("mr") == 2, "The rank should equal 2"
 
     def test_pipeline_evaluation_filtering_with_validation_triples(self):
         """Test if the evaluator's triple filtering with validation triples works as expected using the pipeline."""
@@ -531,4 +530,4 @@ class TestPipelineEvaluationFiltering(unittest.TestCase):
             random_seed=42,
             filter_validation_when_testing=True,
         )
-        assert results.metric_results.arithmetic_mean_rank["both"]["realistic"] == 1, "The rank should equal 1"
+        assert results.metric_results.get_metric("mr") == 1, "The rank should equal 1"
